// Any Device - Utilities - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/common/utils/gsc_util_label.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "gsc_util.h"



/* variables	*************************************************************/

static	int	_width	= 0;
static	int	_level	= 0;



/*****************************************************************************
*
*	Function:	gsc_label_text
*
*	Purpose:
*
*		Print a message introducing a test.
*
*	Arguments:
*
*		label	The test name.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_label_text(const char* label)
{
	int	i;
	int	spaces;

	spaces	= _width - (int) strlen(label);
	spaces	= (spaces < 0) ? 0 : spaces;

	for (i = 0; i < _level; i++)
	{
		printf("  ");
		spaces	-= 2;
	}

	printf("%s: %*s", label, spaces, "");
	fflush(stdout);
}



/*****************************************************************************
*
*	Function:	gsc_label_init
*
*	Purpose:
*
*		Initialize the label code.
*
*	Arguments:
*
*		width	The default width for the label field.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_label_init(int width)
{
	_width	= width;
}



/*****************************************************************************
*
*	Function:	gsc_label_level_dec
*
*	Purpose:
*
*		Decrease the label indenting.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_label_level_dec(void)
{
	_level--;
}



/*****************************************************************************
*
*	Function:	gsc_label_level_inc
*
*	Purpose:
*
*		Increase the label indenting.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_label_level_inc(void)
{
	_level++;
}



/*****************************************************************************
*
*	Function:	gsc_label_fail_text
*
*	Purpose:
*
*		Report a failure with the given text if verbose is enabled.
*
*	Arguments:
*
*		verbose	Work verbosely?
*
*		text	The text to report.
*
*	Returned:
*
*		1		This is an error condition.
*
*****************************************************************************/

int gsc_label_fail_text(int verbose, const char* text)
{
	if ((verbose) && (text))
		printf("FAIL <---  (%s)\n", text);

	return(1);
}



/*****************************************************************************
*
*	Function:	gsc_label_fail_value
*
*	Purpose:
*
*		Report a failure with the given value if verbose is enabled.
*
*	Arguments:
*
*		verbose	Work verbosely?
*
*		value	The value to report.
*
*	Returned:
*
*		1		This is an error condition.
*
*****************************************************************************/

int gsc_label_fail_value(int verbose, long value)
{
	if (verbose)
	{
		printf(	"FAIL <---  (invalid value: %ld, 0x%lX)\n",
			(long) value,
			(long) value);
	}

	return(1);
}



