// Any Device - Utilities - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/common/utils/win32/gsc_os_util_thread.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include "gsc_os_util.h"

#include <process.h>



/* data types	*************************************************************/

typedef struct
{
	U32		(*function)(void* arg);
	void*	arg;
} _thread_data_t;



/*****************************************************************************
*
*	Function:	_thread_func
*
*	Purpose:
*
*		This is an OS specific thread entry function, whose purpose is to
*		call the application's OS independent thread entry function.
*
*	Arguments:
*
*		pvArg	The thread's entry arguemtn.
*
*	Returned:
*
*		TBD
*
*****************************************************************************/

static unsigned WINAPI _thread_func(LPVOID pvArg)
{
	_thread_data_t*	data	= (_thread_data_t*) pvArg;
	DWORD			dw;
	_thread_data_t	tmp		= data[0];

	free(pvArg);
	dw	= (tmp.function)(tmp.arg);
	_endthread();
	return(dw);
}



/*****************************************************************************
*
*	Function:	gsc_os_thread_create
*
*	Purpose:
*
*		Create a thread whose effective entry point and argument are as
*		given.
*
*	Arguments:
*
*		thread		The OS specific thread data is stored here.
*
*		function	The effective entry point.
*
*		arg			The entry argument.
*
*	Returned:
*
*		>= 0		The number of errors seen.
*
*****************************************************************************/

int gsc_os_thread_create(	gsc_os_thread_t*	thread,
							U32					(*function)(void* arg),
							void*				arg)
{
	/*
	*	Under NT I've seen stack overflows that I've investigated but can't
	*	explain so I set the stack size manually to a nice comfortable value.
	*	Under 2000 I didn't see the problem. Don
	*/

	#define	STACK_SIZE	(1L * 1024 * 1024)

	_thread_data_t*	data;
	int				errs;
	unsigned		id;

	for (;;)	// A convenience loop.
	{
		memset(thread, 0, sizeof(gsc_os_thread_t));
		data	= malloc(sizeof(_thread_data_t));

		if (data == NULL)
		{
			errs	= 1;
			break;
		}

		data->function	= function;
		data->arg		= arg;
		thread->handle	= (HANDLE) _beginthreadex(	NULL,
													STACK_SIZE,
													_thread_func,
													data,
													0,
													&id);

		if (thread->handle == NULL)
		{
			free(data);
			errs	= 1;
			break;
		}

		SetThreadPriority(thread->handle, THREAD_PRIORITY_HIGHEST);
		errs	= 0;
		break;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	gsc_os_thread_destroy
*
*	Purpose:
*
*		End a thread in a well behaved manner.
*
*	Arguments:
*
*		thread		The thread to end.
*
*	Returned:
*
*		>= 0		The number of errors seen.
*
*****************************************************************************/

int gsc_os_thread_destroy(gsc_os_thread_t* thread)
{
	U32		status	= GSC_SUCCESS;

	if (thread->handle)
	{
		CloseHandle(thread->handle);
		memset(thread, 0, sizeof(gsc_os_thread_t));
	}

	return(status);
}


