// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_buf_size.c $
// $Rev: 31440 $
// $Date: 2014-11-13 09:44:06 -0600 (Thu, 13 Nov 2014) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY				"Buffer Size"

#define	_1G				(1024L * _1M)
#define	_1K				(1024L)
#define	_1M				(1024L * _1K)
#define	_200M			(200L * _1M)

#define	BUF_SIZE_MAX	_200M
#define	BUF_SIZE_MIN	_1K



/* variables	*************************************************************/

static	HWND		_size_edit;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	char	buf[32];

	if (value < BUF_SIZE_MIN)
		value	= BUF_SIZE_MIN;
	else if (value > BUF_SIZE_MAX)
		value	= BUF_SIZE_MAX;

	gui.buf_size	= value;
	gsc_os_ini_int_set(NULL, NULL, KEY, value);
	gsc_format_long(gui.buf_size, 1024, buf, sizeof(buf));
	SetWindowText(_size_edit, buf);
}



/*****************************************************************************
*
*	Function:	buffer_size_init
*
*	Purpose:
*
*		Initialize things for the Buffer Size option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void buffer_size_init(void)
{
	char	buf[64];
	HWND	control;
	int		i;
	char	max[32];
	char	min[32];

	// The EDIT field.
	_size_edit		= GetDlgItem(gui.dlg, IDC_SIZE_EDIT);
	SendMessage(_size_edit, EM_LIMITTEXT, 12, 0);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, BUF_SIZE_MIN);
	_update_and_save(i);

	// The RANGE field.
	control		= GetDlgItem(gui.dlg, IDC_SIZE_RANGE);
	gsc_format_long(BUF_SIZE_MIN, 1024, min, sizeof(min));
	gsc_format_long(BUF_SIZE_MAX, 1024, max, sizeof(max));
	sprintf(buf, "(range: %s - %s)", min, max);
	SetWindowText(control, buf);
}



/*****************************************************************************
*
*	Function:	buffer_size_refresh
*
*	Purpose:
*
*		Refresh the buffer size controls.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void buffer_size_refresh(void)
{
	EnableWindow(_size_edit, !gui.working);
	SendMessage(_size_edit, WM_PAINT, 0, 0);
}



/*****************************************************************************
*
*	Function:	buffer_size_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Buffer Size controls.
*
*	Arguments:
*
*		wParam	The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void buffer_size_wm_command(WPARAM wParam)
{
	static	int	busy	= 0;

	char		buf[64];
	char		c;
	LRESULT		changed;
	int			code	= HIWORD(wParam);
	int			errs	= 0;
	int			i;
	long		l;
	float		size    = 0;

	if (code == EN_CHANGE)
		changed	= SendMessage( _size_edit, EM_GETMODIFY, 0, 0);
	else
		changed	= 0;

	if (busy == 0)
	{
		busy	= 1;

		if ((code == EN_KILLFOCUS) || (changed))
		{
			GetWindowText(_size_edit, buf, sizeof(buf));
			i	= sscanf(buf, "%ld%c%c", &l, &c, &c);

			switch (i)
			{
				default:
				case -1:
				case 0:
				case 3:	errs++;
						break;

				case 1:	size	= (float) l;
						break;

				case 2:

					switch (c)
					{
						default:	errs++;
									break;
						case 'g':
						case 'G':	size	= _1G * (float) l;
									break;
						case 'k':
						case 'K':	size	= _1K * (float) l;
									break;
						case 'm':
						case 'M':	size	= _1M * (float) l;
									break;
					}
			}

			if ((size < BUF_SIZE_MIN) || (size > BUF_SIZE_MAX))
				errs++;

			gui.buf_size_invalid	= errs ? TRUE : FALSE;
			start_refresh();

			if ((errs == 0) && (code == EN_KILLFOCUS))
				_update_and_save((int) (long) size);
		}

		busy	= 0;
	}
}


