// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_dev_list.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "gsc_os_util.h"
#include "gsc_util.h"
#include "hpdi32_util.h"



/* #defines	*****************************************************************/

#define	KEY		"Device Index"



/* variables	*************************************************************/

static	HWND	_device_list;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	if (value >= gui.dev_qty)
		value	= gui.dev_qty - 1;

	if (value < 0)
		value	= 0;

	gui.dev_index	= value;
	gsc_os_ini_int_set(NULL, NULL, KEY, gui.dev_index);
}



/*****************************************************************************
*
*	Function:	device_list_init
*
*	Purpose:
*
*		Initialize things for the Device List option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void device_list_init(void)
{
	U32		arg;
	char	buf[128];
	U8		count;
	int		i;
	U32		stat;
	U32		status;

	for (;;)	// A convenience loop.
	{
		_device_list	= GetDlgItem(gui.dlg, IDC_DEVICE_LIST);
		stat			= hpdi32_api_status(&status, &arg, HPDI32_API_VERSION);

		if (stat != GSC_SUCCESS)
		{
			gsc_label_text("API Status");
			label_fail_func(1, "hpdi32_api_status", stat);
			count	= 0;
			break;
		}

		if (status != GSC_SUCCESS)
		{
			gsc_label_text("API Status");
			sprintf(buf, "hpdi32_api_status(): status 0x%lX, arg 0x%lX", (long) status, (long) arg);
			gsc_label_fail_text(1, buf);
			count	= 0;
			break;
		}

		status	= hpdi32_board_count(&count);

		if (status != GSC_SUCCESS)
		{
			gsc_label_text("Device Count");
			label_fail_func(1, "hpdi32_board_count", status);
			count	= 0;
			break;
		}

		gui.dev_qty	= count;
		gsc_os_ini_int_get(NULL, NULL, KEY, &i, 0);
		_update_and_save(i);

		// Update the device selection list box.

		if (gui.dev_qty <= 0)
		{
			// There are no device present.
			EnableWindow(_device_list, FALSE);
			SendMessage(_device_list, WM_SETREDRAW, 0, 0);
			SendMessage(_device_list, LB_RESETCONTENT, 0, 0);
			SendMessage(_device_list, LB_ADDSTRING, 0, (LPARAM) "None");
			SendMessage(_device_list, LB_SETTOPINDEX, (WPARAM) 0, 0);
			SendMessage(_device_list, LB_SETCURSEL, (WPARAM) -1, 0);
			SendMessage(_device_list, WM_SETREDRAW, 1, 0);
		}
		else
		{
			// List the devices present.
			EnableWindow(_device_list, TRUE);
			SendMessage(_device_list, WM_SETREDRAW, 0, 0);
			SendMessage(_device_list, LB_RESETCONTENT, 0, 0);

			for (i = 0; i < gui.dev_qty; i++)
			{
				sprintf(buf, "HPDI32 #%d", i);
				SendMessage(_device_list, LB_ADDSTRING, 0, (LPARAM) buf);
			}

			SendMessage(_device_list, LB_SETCURSEL, (WPARAM) gui.dev_index, 0);
			SendMessage(_device_list, WM_SETREDRAW, 1, 0);
		}

		break;
	}
}



/*****************************************************************************
*
*	Function:	device_list_refresh
*
*	Purpose:
*
*		Refresh the Device List controls.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void device_list_refresh(void)
{
	EnableWindow(_device_list, !gui.working);
	SendMessage(_device_list, WM_PAINT, 0, 0);
}



/*****************************************************************************
*
*	Function:	device_list_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Device List controls.
*
*	Arguments:
*
*		wParam	The WORD sized parameter received with the message.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void device_list_wm_command(WPARAM wParam)
{
	int		code	= HIWORD(wParam);
	LRESULT	i;

	if (code == LBN_SELCHANGE)
	{
		i	= SendMessage(_device_list, LB_GETCURSEL, 0, 0);
		_update_and_save((int) i);
	}
}


