// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_edit_int.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"


/* variables	**************************************************************/

static	const char*	_info;
static	int			_max;
static	int			_min;
static	const char*	_name;
static	const char*	_units;
static	int*		_value;



/*****************************************************************************
*
*	Function:	_idc_edit
*
*	Purpose:
*
*		Respond to activity in the edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_edit(HWND dlg, WPARAM wParam)
{
	char	buf[32];
	char	c;
	int		code	= HIWORD(wParam);
	HWND	control;
	int		i;
	int		v;

	if (code == EN_CHANGE)
	{
		control	= GetDlgItem(dlg, IDC_EDIT);
		GetWindowText(control, buf, sizeof(buf));
		control	= GetDlgItem(dlg, IDOK);
		i	= sscanf(buf, "%d%c", &v, &c);

		if ((i != 1) || (v < _min) || (v > _max))
			EnableWindow(control, 0);
		else
			EnableWindow(control, 1);
	}
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg)
{
	char	buf[32];
	HWND	control;

	control	= GetDlgItem(dlg, IDC_EDIT);
	GetWindowText(control, buf, sizeof(buf));
	sscanf(buf, "%d", _value);
	EndDialog(dlg, 0);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	int			id		= LOWORD(wParam);
	LRESULT		ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_EDIT:

			_idc_edit(dlg, wParam);
			break;

		case IDCANCEL:

			EndDialog(dlg, 0);
			break;

		case IDOK:

			_idok(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg)
{
	char	buf[1024];
	HWND	control;
	int		v;

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) gui.icon_sm);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) gui.icon_lg);

	GetWindowText(dlg, buf, sizeof(buf));
	strcat(buf, ": ");
	strcat(buf, _name);
	SetWindowText(dlg, buf);

	control	= GetDlgItem(dlg, IDC_EDIT);
	SendMessage(control, EM_LIMITTEXT, 10, 0);
	v		= _value[0];
	v		= (v < _min) ? _min : v;
	v		= (v > _max) ? _max : v;
	sprintf(buf, "%d", v);
	SetWindowText(control, buf);

	control	= GetDlgItem(dlg, IDC_UNITS);
	SetWindowText(control, _units);

	control	= GetDlgItem(dlg, IDC_RANGE);
	sprintf(buf, "%d - %d", _min, _max);
	SetWindowText(control, buf);

	control	= GetDlgItem(dlg, IDC_INFO);
	SetWindowText(control, _info);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			EndDialog(dlg, 0);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	gui_edit_int
*
*	Purpose:
*
*		Open a dialog box to edit an integer.
*
*	Arguments:
*
*		name		The name of the value being edited.
*
*		value		The value to edit is recorded here.
*
*		min			This is the minimum acceptable value.
*
*		max			This is the maximum acceptible value.
*
*		units		This is the units that the value represents.
*
*		info		This is information about the item being edited.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gui_edit_int(
	const char*	name,
	int*		value,
	int			min,
	int			max,
	const char*	units,
	const char*	info)
{
	int	i	= min;

	_name	= name ? name : "ERROR";
	_value	= value ? value : &i;
	_min	= min;
	max		= (max < 0) ? max & 0x7FFFFFFF : max;
	_max	= (max < min) ? min : max;
	_units	= units ? units : "ERROR";
	_info	= info ? info : "";

	DialogBox(	gui.instance,
				MAKEINTRESOURCE(IDD_EDIT_INT),
				gui.dlg,
				_dlg_proc);
}


