// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_favor_tx.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY	"HPDI32_MISC_FAVOR_TX"



/* variables	*************************************************************/

static	HMENU		_menu;
static	const char*	_name;
static	int			_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	if (value == HPDI32_MISC_FAVOR_TX_DISABLE)
	{
		_name		= "HPDI32_MISC_FAVOR_TX_DISABLE";
		_setting	= HPDI32_MISC_FAVOR_TX_DISABLE;
		gui_menu_add_item_text(_menu, IDC_FAVOR_TX_NO);
		CheckMenuItem(_menu, IDC_FAVOR_TX_NO, MF_BYCOMMAND | MF_CHECKED);
		CheckMenuItem(_menu, IDC_FAVOR_TX_YES, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else
	{
		_name		= "HPDI32_MISC_FAVOR_TX_ENABLE";
		_setting	= HPDI32_MISC_FAVOR_TX_ENABLE;
		gui_menu_add_item_text(_menu, IDC_FAVOR_TX_YES);
		CheckMenuItem(_menu, IDC_FAVOR_TX_NO, MF_BYCOMMAND | MF_UNCHECKED);
		CheckMenuItem(_menu, IDC_FAVOR_TX_YES, MF_BYCOMMAND | MF_CHECKED);
	}

	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
}



/*****************************************************************************
*
*	Function:	favor_tx_init
*
*	Purpose:
*
*		Initialize things for the Favor Tx option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void favor_tx_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	_menu	= gui_menu_locate_parent_by_id(_menu, IDC_FAVOR_TX_NO);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, HPDI32_MISC_FAVOR_TX_DEFAULT);
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	favor_tx_refresh
*
*	Purpose:
*
*		Refresh the Favor Tx menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void favor_tx_refresh(void)
{
	if (gui.working)
	{
		EnableMenuItem(_menu, IDC_FAVOR_TX_NO, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_FAVOR_TX_YES, MF_BYCOMMAND | MF_GRAYED);
	}
	else
	{
		EnableMenuItem(_menu, IDC_FAVOR_TX_NO, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_FAVOR_TX_YES, MF_BYCOMMAND | MF_ENABLED);
	}
}



/*****************************************************************************
*
*	Function:	favor_tx_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Favor Tx control or menu items.
*
*	Arguments:
*
*		id		The id of the control or menu item selected.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void favor_tx_wm_command(int id)
{
	switch (id)
	{
		default:
		case IDC_FAVOR_TX_NO:

			id	= HPDI32_MISC_FAVOR_TX_DISABLE;
			break;

		case IDC_FAVOR_TX_YES:

			id	= HPDI32_MISC_FAVOR_TX_ENABLE;
			break;
	}

	_update_and_save(id);
}



/*****************************************************************************
*
*	Function:	favor_tx_config
*
*	Purpose:
*
*		Configure the given device's Favor Tx parameter.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

int favor_tx_config(void* dev)
{
	int	errs;
	U32	status;

	gsc_label_text(KEY);
	status	= HPDI32_MISC_FAVOR_TX__SET(dev, _setting);
	errs	= label_fail_func(1, "HPDI32_MISC_FAVOR_TX__SET", status);

	if (errs == 0)
		printf("PASS  (%s)\n", _name);

	return(errs);
}
