// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_rx_operation.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "proc.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY			"Rx Operation"
#define	KEY_FILE	"Rx File"



/* variables	*************************************************************/

static	HMENU		_menu;
static	const char*	_name;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	switch (value)
	{
		default:
		case RX_OP_INCREMENT:

			_name		= "Increment";
			gui.rx_op	= RX_OP_INCREMENT;
			CheckMenuItem(_menu, IDC_RX_OP_INCREMENT, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_RX_OP_RECEIVE, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_RX_OP_LOAD, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case RX_OP_RECEIVE:

			_name		= "Receive";
			gui.rx_op	= RX_OP_RECEIVE;
			CheckMenuItem(_menu, IDC_RX_OP_INCREMENT, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_RX_OP_RECEIVE, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_RX_OP_LOAD, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case RX_OP_LOAD:

			_name		= "Load";
			gui.rx_op	= RX_OP_LOAD;
			CheckMenuItem(_menu, IDC_RX_OP_INCREMENT, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_RX_OP_RECEIVE, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_RX_OP_LOAD, MF_BYCOMMAND | MF_CHECKED);
			break;
	}

	gui_menu_add_text(_menu, _name);
	gsc_os_ini_int_set(NULL, NULL, KEY, gui.rx_op);
	gsc_os_ini_str_set(NULL, NULL, KEY_FILE, gui.rx_file);
}



/*****************************************************************************
*
*	Function:	_select_rx_file
*
*	Purpose:
*
*		Prompt for the file to be read as the source for receive data.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0		No selection was made.
*		else	A selection was made.
*
*****************************************************************************/

static int _select_rx_file(void)
{
	char			dir[1024];
	int				i;
	OPENFILENAME	ofn;

	GetCurrentDirectory(sizeof(dir), dir);
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= gui.dlg;
	ofn.lpstrFile		= gui.rx_file;
	ofn.nMaxFile		= sizeof(gui.rx_file);
	ofn.lpstrInitialDir	= dir;
	ofn.lpstrTitle		= "Receive Data Source File";
	ofn.Flags			= OFN_ENABLESIZING
						| OFN_FILEMUSTEXIST
						| OFN_PATHMUSTEXIST
						| OFN_HIDEREADONLY;
	i					= GetOpenFileName(&ofn);
	return(i);
}



/*****************************************************************************
*
*	Function:	rx_operation_init
*
*	Purpose:
*
*		Initialize things for the Rx Operation option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void rx_operation_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	_menu	= gui_menu_locate_parent_by_id(_menu, IDC_RX_OP_INCREMENT);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, RX_OP_INCREMENT);
	gsc_os_ini_str_get(NULL, NULL, KEY_FILE, gui.rx_file, sizeof(gui.rx_file));
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	rx_operation_refresh
*
*	Purpose:
*
*		Refresh the Rx Operation menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void rx_operation_refresh(void)
{
	if (gui.working)
	{
		EnableMenuItem(_menu, IDC_RX_OP_INCREMENT, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_RX_OP_RECEIVE, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_RX_OP_LOAD, MF_BYCOMMAND | MF_GRAYED);
	}
	else
	{
		EnableMenuItem(_menu, IDC_RX_OP_INCREMENT, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_RX_OP_RECEIVE, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_RX_OP_LOAD, MF_BYCOMMAND | MF_ENABLED);
	}
}



/*****************************************************************************
*
*	Function:	rx_operation_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Rx Operation control or
*		menu items.
*
*	Arguments:
*
*		id		The id of the control or menu item selected.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void rx_operation_wm_command(int id)
{
	int	i;

	switch (id)
	{
		default:
		case IDC_RX_OP_INCREMENT:

			id	= RX_OP_INCREMENT;
			_update_and_save(id);
			break;

		case IDC_RX_OP_RECEIVE:

			id	= RX_OP_RECEIVE;
			_update_and_save(id);
			break;

		case IDC_RX_OP_LOAD:

			i	= _select_rx_file();

			if (i)
			{
				id	= RX_OP_LOAD;
				_update_and_save(id);
			}

			break;
	}
}


