// HPDI32 - Utilities - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/utils/util_reg.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <stdio.h>

#include "hpdi32_util.h"
#include "hpdi32_dsl.h"



/* #defines	*************************************************************/

#define	_GSC_REG(a)	"GSC " #a, HPDI32_##a



/* variables	*************************************************************/

static const gsc_util_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(FRR)		},
	{ _GSC_REG(BCR)		},
	{ _GSC_REG(BSR)		},
	{ _GSC_REG(TAR)		},
	{ _GSC_REG(RAR)		},
	{ _GSC_REG(FSR)		},
	{ _GSC_REG(FDR)		},
	{ _GSC_REG(TSVLCR)	},
	{ _GSC_REG(TLVLCR)	},
	{ _GSC_REG(TLILCR)	},
	{ _GSC_REG(RSCR)	},
	{ _GSC_REG(RLCR)	},
	{ _GSC_REG(ICR)		},
	{ _GSC_REG(ISR)		},
	{ _GSC_REG(TCDR)	},
	{ _GSC_REG(TFSR)	},
	{ _GSC_REG(RFSR)	},
	{ _GSC_REG(TFWR)	},
	{ _GSC_REG(RFWR)	},
	{ _GSC_REG(IELR)	},
	{ _GSC_REG(IHLR)	},
	{ NULL, 0			}
};



/*****************************************************************************
*
*	Function:	_reg_list
*
*	Purpose:
*
*		List the given registers and their values.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*		list	The register list to display.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _reg_list(void* dev, const gsc_util_reg_def_t* list)
{
	int	errs	= 0;
	int	status;
	U32	v;

	printf("\n\n");

	for (; list->name; list++)
	{
		status	= hpdi32_reg_read(dev, list->reg, &v);
		printf("%-10s 0x%08lX", list->name, (long) v);

		if (status)
		{
			errs++;
			printf("  FAIL <---  (hpdi32_reg_read()");
		}

		printf("\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_reg_name
*
*	Purpose:
*
*		Return the name of the given register.
*
*	Arguments:
*
*		reg		The register id.
*
*		list	The list of registers to check.
*
*	Returned:
*
*		!NULL	The name of the given register.
*		NULL	The register id isn't in the given list.
*
*****************************************************************************/

static const char* _reg_name(U32 reg, const gsc_util_reg_def_t* list)
{
	const char*	ptr	= NULL;

	for (; list->name; list++)
	{
		if (list->reg == reg)
		{
			ptr	= list->name;
			break;
		}
	}

	return(ptr);
}



/*****************************************************************************
*
*	Function:	dev_reg_mod
*
*	Purpose:
*
*		Put a wrapper around hpdi32_reg_mod().
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*		reg		The register to access.
*
*		value	The value to apply.
*
*		mask	The set of bits to modify.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

int dev_reg_mod(void* dev, U32 reg, U32 value, U32 mask, int verbose)
{
	int	errs;
	U32	status;

	status	= hpdi32_reg_mod(dev, reg, value, mask);
	errs	= label_fail_func(verbose, "hpdi32_reg_mod", status);
	return(errs);
}



/*****************************************************************************
*
*	Function:	dev_reg_read
*
*	Purpose:
*
*		Put a wrapper around hpdi32_reg_read().
*
*	Arguments:
*
*		dev		The handle for the device of interest.
*
*		reg		The register to access.
*
*		value	The value read goes here.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

int dev_reg_read(void* dev, U32 reg, U32* value, int verbose)
{
	int	errs;
	U32	status;

	status	= hpdi32_reg_read(dev, reg, value);
	errs	= label_fail_func(verbose, "hpdi32_reg_read", status);
	return(errs);
}



/*****************************************************************************
*
*	Function:	dev_reg_write
*
*	Purpose:
*
*		Put a wrapper around the hpdi32_reg_write() call.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*		reg		The register to access.
*
*		value	The value to write.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

int dev_reg_write(void* dev, U32 reg, U32 value, int verbose)
{
	int	errs;
	U32	status;

	status	= hpdi32_reg_write(dev, reg, value);
	errs	= label_fail_func(verbose, "hpdi32_reg_write", status);
	return(errs);
}



/*****************************************************************************
*
*	Function:	reg_list
*
*	Purpose:
*
*		List the given registers and their values.
*
*	Arguments:
*
*		dev		The handle of the device to access.
*
*		pci		List the PCI registers?
*
*		plx		List the PLX registers?
*
*		gsc		List the GSC registers?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

int reg_list(void* dev, int pci, int plx, int gsc)
{
	int	errs	= 0;

	if (pci)
		errs	+= _reg_list(dev, gsc_pci9080_pci);

	if (plx)
		errs	+= _reg_list(dev, gsc_pci9080_plx);

	if (gsc)
		errs	+= _reg_list(dev, _gsc);

	return(errs);
}



/*****************************************************************************
*
*	Function:	reg_name
*
*	Purpose:
*
*		Return the name of the given register.
*
*	Arguments:
*
*		reg		The register id.
*
*	Returned:
*
*		!NULL	Either the name or an error string.
*
*****************************************************************************/

const char* reg_name(U32 reg)
{
	static	char	buf[32];
	const char*		ptr;

	ptr	= _reg_name(reg, _gsc);

	if (ptr == NULL)
		ptr	= _reg_name(reg, gsc_pci9080_pci);

	if (ptr == NULL)
		ptr	= _reg_name(reg, gsc_pci9080_plx);

	if (ptr == NULL)
	{
		sprintf(buf, "0x%lX", (long) reg);
		ptr	= buf;
	}

	return(ptr);
}


