// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/arrow.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Arrow styled controls.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "arrow.h"
#include "gui.h"



/* #defines	*****************************************************************/

#define	HORIZONTAL_MID(r)	(((r).left + (r).right) / 2)
#define	VERTICAL_MID(r)		(((r).top + (r).bottom) / 2)



/* typedefs	*****************************************************************/

typedef struct
{
	HBRUSH	brush;
	int		enable;
	POINT	list[5];
	HPEN	pen;
} head_t;

typedef struct
{
	int		enable;
	POINT	from;
	HPEN	pen;
	HPEN	pen_bg;
	POINT	to;
} line_t;

typedef struct
{
	HBRUSH	brush;
	int		enable;
	HPEN	pen;
	RECT	rect;
} rect_t;

typedef struct
{
	int			aligned;
	COLORREF	color_fg;
	int			enable;
	HFONT		font;
	int			height;
	char		text[64];
	int			x;
	int			y;
} text_t;

typedef struct
{
	rect_t		background;
	device_t*	device;
	head_t		head;
	line_t		line1;
	line_t		line2;
	line_t		line3;
	line_t		line4;
	line_t		line5;
	line_t		line6;
	rect_t		rect;
	RECT		text;
	text_t		text1;
	text_t		text2;
	int			tmAscent;
	int			tmDescent;
} arrow_t;



/*****************************************************************************
*
*	Function:	_head_paint
*
*	Purpose:
*
*		Draw an arrow head per the given structure.
*
*	Arguments:
*
*		dc			The device drawing context.
*
*		data		The line data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _head_paint(HDC dc, head_t* data)
{
	HBRUSH	brush;
	HPEN	pen;

	if (data->enable)
	{
		brush	= SelectObject(dc, data->brush);
		pen		= SelectObject(dc, data->pen);
		Polygon(dc, data->list, ARRAY_ELEMENTS(data->list));
		SelectObject(dc, brush);
		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_line_paint
*
*	Purpose:
*
*		Draw a line per the given structure.
*
*	Arguments:
*
*		dc			The device drawing context.
*
*		data		The line data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _line_paint(HDC dc, line_t* data)
{
	HPEN	pen;

	if (data->enable)
	{
		if (data->pen_bg)
			pen	= SelectObject(dc, data->pen_bg);
		else
			pen	= SelectObject(dc, data->pen);

		MoveToEx(dc, data->from.x, data->from.y, NULL);
		LineTo(dc, data->to.x, data->to.y);
		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_rect_paint
*
*	Purpose:
*
*		Draw a filled rectangle per the given structure.
*
*	Arguments:
*
*		dc			The device drawing context.
*
*		data		The rectangle data to use.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _rect_paint(HDC dc, rect_t* data)
{
	HBRUSH	brush;
	HPEN	pen;

	if (data->enable)
	{
		brush	= SelectObject(dc, data->brush);
		pen		= SelectObject(dc, data->pen);
		Rectangle(	dc,
					data->rect.left,
					data->rect.top,
					RECT_WIDTH(data->rect),
					RECT_HEIGHT(data->rect));
		SelectObject(dc, brush);
		SelectObject(dc, pen);
	}
}



/*****************************************************************************
*
*	Function:	_text_height
*
*	Purpose:
*
*		Get the height of the given text.
*
*	Arguments:
*
*		data		The control specific data we track.
*
*		src			The string to examine.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _text_height(arrow_t* data, text_t* text)
{
	int	i;

	text->height	= data->tmAscent;

	for (i = 0; text->text[i]; i++)
	{
		if (strchr("gjpqy", text->text[i]))
		{
			text->height	+= data->tmDescent;
			break;
		}
	}
}



/*****************************************************************************
*
*	Function:	_text_align
*
*	Purpose:
*
*		Align the text in the control
*
*	Arguments:
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _text_align(arrow_t* data)
{
	if (((data->text1.enable) && (data->text2.enable)) &&
		((data->text1.aligned == FALSE) || (data->text2.aligned == FALSE)))
	{
		_text_height(data, &data->text1);
		_text_height(data, &data->text2);

		data->text1.x		= HORIZONTAL_MID(data->text);
		data->text1.y		= (RECT_HEIGHT(data->text)
							- data->text1.height - data->text2.height) / 2;
		data->text1.aligned	= TRUE;
		data->text2.x		= data->text1.x;
		data->text2.y		= data->text1.y + data->text2.height;
		data->text2.aligned	= TRUE;
	}
	else if ((data->text1.enable) && (data->text1.aligned == FALSE))
	{
		_text_height(data, &data->text1);
		data->text1.x		= HORIZONTAL_MID(data->text);
		data->text1.y		= (RECT_HEIGHT(data->text) - data->text1.height) / 2;
		data->text1.aligned	= TRUE;
	}
	else if ((data->text2.enable) && (data->text2.aligned == FALSE))
	{
		_text_height(data, &data->text1);
		data->text2.x		= HORIZONTAL_MID(data->text);
		data->text2.y		= (RECT_HEIGHT(data->text) - data->text2.height) / 2;
		data->text2.aligned	= TRUE;
	}
}



/*****************************************************************************
*
*	Function:	_text_paint
*
*	Purpose:
*
*		Process WM_PAINT messages.
*
*	Arguments:
*
*		data	The control specific data we track.
*
*		dc		The device context to paint with.
*
*		text	The structure to process.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _text_paint(arrow_t* data, HDC dc, text_t* text)
{
	UINT		align;
	char		buf[sizeof(text->text) + 10];
	COLORREF	color_bk;
	COLORREF	color_fg;
	HFONT		font;
	UINT		len;

	if (text->enable)
	{
		sprintf(buf, " %s ", text->text);
		len			= (int) strlen(buf);
		align		= SetTextAlign(dc, TA_CENTER | TA_TOP);
		color_bk	= SetBkColor(dc, data->device->paint.btnhighlt.color);
		color_fg	= SetTextColor(dc, text->color_fg);
		font		= SelectObject(dc, text->font);
		ExtTextOut(	dc,
					text->x,
					text->y,
					ETO_CLIPPED,
					&data->text,
					buf,
					len,
					NULL);
		SelectObject(dc, font);
		SetBkColor(dc, color_bk);
		SetTextAlign(dc, align);
		SetTextColor(dc, color_fg);
	}
}



/*****************************************************************************
*
*	Function:	_config_none
*
*	Purpose:
*
*		Disable all drawing items in the arrow.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_none(HWND control, arrow_t* data)
{
	data->background.enable	= TRUE;
	data->head.enable		= FALSE;
	data->line1.enable		= FALSE;
	data->line1.pen_bg		= NULL;
	data->line2.enable		= FALSE;
	data->line2.pen_bg		= NULL;
	data->line3.enable		= FALSE;
	data->line3.pen_bg		= NULL;
	data->line4.enable		= FALSE;
	data->line4.pen_bg		= NULL;
	data->line5.enable		= FALSE;
	data->line5.pen_bg		= NULL;
	data->line6.enable		= FALSE;
	data->line6.pen_bg		= NULL;
	data->rect.enable		= FALSE;
	data->text1.aligned		= FALSE;
	data->text2.aligned		= FALSE;
	GetClientRect(control, &data->text);
}



/*****************************************************************************
*
*	Function:	_config_horz_1
*
*	Purpose:
*
*		Configure the arrow for a single horizontal line.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_horz_1(HWND control, arrow_t* data)
{
	RECT	rect;
	int		y;

	_config_none(control, data);
	GetClientRect(control, &rect);
	y	= VERTICAL_MID(rect);

	data->line1.enable	= TRUE;
	data->line1.from.x	= rect.left;
	data->line1.from.y	= y;
	data->line1.to.x	= rect.right;
	data->line1.to.y	= y;
}



/*****************************************************************************
*
*	Function:	_config_horz_2
*
*	Purpose:
*
*		Configure the arrow for a pair of horizontal lines.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_horz_2(HWND control, arrow_t* data)
{
	int		dy	= (GetSystemMetrics(SM_CYSMICON) * 3) / 20;
	RECT	rect;
	int		y;

	_config_none(control, data);
	GetClientRect(control, &rect);
	y	= VERTICAL_MID(rect);

	data->line1.enable	= TRUE;
	data->line1.from.x	= rect.left;
	data->line1.from.y	= y - dy;
	data->line1.to.x	= rect.right;
	data->line1.to.y	= y - dy;

	data->line2.enable	= TRUE;
	data->line2.from.x	= rect.left;
	data->line2.from.y	= y + dy;
	data->line2.to.x	= rect.right;
	data->line2.to.y	= y + dy;
}



/*****************************************************************************
*
*	Function:	_config_rect_solid
*
*	Purpose:
*
*		Configure the arrow for a filled rectangle.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_rect_solid(HWND control, arrow_t* data)
{
	_config_none(control, data);
	data->background.enable	= FALSE;
	data->rect.enable		= TRUE;
	GetClientRect(control, &data->rect.rect);
}



/*****************************************************************************
*
*	Function:	_config_vert_1
*
*	Purpose:
*
*		Configure the arrow for a single vertical line.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_vert_1(HWND control, arrow_t* data)
{
	RECT	rect;
	int		x;

	_config_none(control, data);
	GetClientRect(control, &rect);
	x	= HORIZONTAL_MID(rect);

	data->line1.enable	= TRUE;
	data->line1.from.x	= x;
	data->line1.from.y	= rect.top;
	data->line1.to.x	= x;
	data->line1.to.y	= rect.bottom;
}



/*****************************************************************************
*
*	Function:	_config_vert_2
*
*	Purpose:
*
*		Configure the arrow for a pair of vertical lines.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_vert_2(HWND control, arrow_t* data)
{
	int		dx	= (GetSystemMetrics(SM_CXSMICON) * 3) / 20;
	RECT	rect;
	int		x;

	_config_none(control, data);
	GetClientRect(control, &rect);
	x	= HORIZONTAL_MID(rect);

	data->line1.enable	= TRUE;
	data->line1.from.x	= x - dx;
	data->line1.from.y	= rect.top;
	data->line1.to.x	= x - dx;
	data->line1.to.y	= rect.bottom;

	data->line2.enable	= TRUE;
	data->line2.from.x	= x + dx;
	data->line2.from.y	= rect.top;
	data->line2.to.x	= x + dx;
	data->line2.to.y	= rect.bottom;
}



/*****************************************************************************
*
*	Function:	_config_lower_right_2
*
*	Purpose:
*
*		Configure the arrow for a double lower right corner.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_lower_right_2(HWND control, arrow_t* data)
{
	arrow_t	tmp;

	_config_vert_2(control, data);
	tmp	= data[0];
	_config_horz_2(control, data);

	data->line1.to.x	= tmp.line1.to.x;
	data->line2.to.x	= tmp.line2.to.x;
	data->line3			= tmp.line1;
	data->line4			= tmp.line2;
	data->line3.to.y	= data->line1.from.y + 1;
	data->line4.to.y	= data->line2.from.y + 1;
}



/*****************************************************************************
*
*	Function:	_config_upper_right_2
*
*	Purpose:
*
*		Configure the arrow for a double upper right corner.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_upper_right_2(HWND control, arrow_t* data)
{
	arrow_t	tmp;

	_config_vert_2(control, data);
	tmp	= data[0];
	_config_horz_2(control, data);

	data->line1.to.x	= tmp.line2.to.x;
	data->line2.to.x	= tmp.line1.to.x;
	data->line3			= tmp.line1;
	data->line4			= tmp.line2;
	data->line3.from.y	= data->line2.from.y;
	data->line4.from.y	= data->line1.from.y;
}



/*****************************************************************************
*
*	Function:	_config_tee_right_2
*
*	Purpose:
*
*		Configure the arrow for a double "T" whose bottom is at right.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_tee_right_2(HWND control, arrow_t* data)
{
	arrow_t	tmp;

	_config_horz_2(control, data);
	tmp	= data[0];
	_config_vert_2(control, data);

	data->line3			= data->line2;
	data->line2.to.y	= tmp.line1.from.y;
	data->line3.from.y	= tmp.line2.from.y;
	data->line4			= tmp.line1;
	data->line4.from.x	= data->line2.from.x;
	data->line5			= tmp.line2;
	data->line5.from.x	= data->line2.from.x;
}



/*****************************************************************************
*
*	Function:	_config_tee_right_lower_left_2
*
*	Purpose:
*
*		Configure the arrow for a double "T" whose bottom is at right, and
*		appears as if the "T" is grayed and Lower Left corner portion is not.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_tee_right_lower_left_2(HWND control, arrow_t* data)
{
	_config_tee_right_2(control, data);
	data->line6	= data->line1;
	data->line1.to.y	= data->line3.from.y;
	data->line3.pen_bg	= data->device->paint.btnface.pen;
	data->line5.from.x	= data->line1.from.x;
	data->line6.from.y	= data->line3.from.y + 1;
	data->line6.pen_bg	= data->device->paint.btnface.pen;
}



/*****************************************************************************
*
*	Function:	_config_tee_right_upper_left_2
*
*	Purpose:
*
*		Configure the arrow for a double "T" whose bottom is at right, and
*		appears as if the "T" is grayed and Lower Left corner portion is not.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_tee_right_upper_left_2(HWND control, arrow_t* data)
{
	_config_tee_right_2(control, data);
	data->line6	= data->line1;
	data->line1.from.y	= data->line4.from.y;
	data->line2.pen_bg	= data->device->paint.btnface.pen;
	data->line4.from.x	= data->line1.from.x;
	data->line6.to.y	= data->line4.from.y;
	data->line6.pen_bg	= data->device->paint.btnface.pen;
}



/*****************************************************************************
*
*	Function:	_config_arrow_bottom_1
*
*	Purpose:
*
*		Configure the arrow for a single horizontal line pointing down.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_bottom_1(HWND control, arrow_t* data, int dx, int dy)
{
	int	x;
	int	x1;
	int	x2;
	int	y1;
	int	y2;

	_config_vert_1(control, data);

	x	= data->line1.to.x;
	x1	= x + dx;
	x2	= x - dx;
	y1	= data->line1.to.y - 1;
	y2	= y1 - dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x;
	data->head.list[0].y	= y2;
	data->head.list[1].x	= x1;
	data->head.list[1].y	= y2;
	data->head.list[2].x	= x;
	data->head.list[2].y	= y1;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x;
	data->head.list[4].y	= y2;

	data->line1.to.y		-= dy;
	data->text.bottom		= data->line1.to.y;
}



/*****************************************************************************
*
*	Function:	_config_arrow_bottom_2
*
*	Purpose:
*
*		Configure the arrow for a pair of horizontal line pointing down.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_bottom_2(HWND control, arrow_t* data, int dx, int dy)
{
	RECT	rect;
	int		x;
	int		x1;
	int		x2;
	int		y1;
	int		y2;

	_config_vert_1(control, data);

	GetClientRect(control, &rect);
	x	= HORIZONTAL_MID(rect);
	x1	= x + dx;
	x2	= x - dx;
	y1	= data->line1.to.y - 1;
	y2	= y1 - dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x;
	data->head.list[0].y	= y2;
	data->head.list[1].x	= x1;
	data->head.list[1].y	= y2;
	data->head.list[2].x	= x;
	data->head.list[2].y	= y1;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x;
	data->head.list[4].y	= y2;

	data->line1.to.y		-= dy;
	data->line2.to.y		-= dy;
	data->text.bottom		= data->line1.to.y;
}



/*****************************************************************************
*
*	Function:	_config_arrow_left_1
*
*	Purpose:
*
*		Configure the arrow for a single horizontal line pointing left.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_left_1(HWND control, arrow_t* data, int dx, int dy)
{
	int	x1;
	int	x2;
	int	y;
	int	y1;
	int	y2;

	_config_horz_1(control, data);

	x1	= data->line1.from.x;
	x2	= x1 + dx;
	y	= data->line1.from.y;
	y1	= y + dy;
	y2	= y - dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x2;
	data->head.list[0].y	= y;
	data->head.list[1].x	= x2;
	data->head.list[1].y	= y1;
	data->head.list[2].x	= x1;
	data->head.list[2].y	= y;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x2;
	data->head.list[4].y	= y;

	data->line1.from.x		+= dx;
	data->text.left			= data->line1.from.x;
}



/*****************************************************************************
*
*	Function:	_config_arrow_left_2
*
*	Purpose:
*
*		Configure the arrow for a pair of horizontal lines pointing left.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_left_2(HWND control, arrow_t* data, int dx, int dy)
{
	RECT	rect;
	int		x1;
	int		x2;
	int		y;
	int		y1;
	int		y2;

	_config_horz_2(control, data);

	GetClientRect(control, &rect);
	x1	= data->line1.from.x;
	x2	= x1 + dx;
	y	= VERTICAL_MID(rect);
	y1	= y + dy;
	y2	= y - dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x2;
	data->head.list[0].y	= y;
	data->head.list[1].x	= x2;
	data->head.list[1].y	= y1;
	data->head.list[2].x	= x1;
	data->head.list[2].y	= y;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x2;
	data->head.list[4].y	= y;

	data->line1.from.x		+= dx;
	data->line2.from.x		+= dx;
	data->text.left			= data->line1.from.x;
}



/*****************************************************************************
*
*	Function:	_config_arrow_right_1
*
*	Purpose:
*
*		Configure the arrow for a single horizontal line pointing right.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_right_1(HWND control, arrow_t* data, int dx, int dy)
{
	int	x1;
	int	x2;
	int	y;
	int	y1;
	int	y2;

	_config_horz_1(control, data);

	x1	= data->line1.to.x - 1;
	x2	= x1 - dx;
	y	= data->line1.to.y;
	y1	= y + dy;
	y2	= y - dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x2;
	data->head.list[0].y	= y;
	data->head.list[1].x	= x2;
	data->head.list[1].y	= y1;
	data->head.list[2].x	= x1;
	data->head.list[2].y	= y;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x2;
	data->head.list[4].y	= y;

	data->line1.to.x		-= dx;
	data->text.right		= data->line1.to.x;
}



/*****************************************************************************
*
*	Function:	_config_arrow_right_2
*
*	Purpose:
*
*		Configure the arrow for a pair of horizontal lines pointing right.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_right_2(HWND control, arrow_t* data, int dx, int dy)
{
	RECT	rect;
	int		x1;
	int		x2;
	int		y;
	int		y1;
	int		y2;

	_config_horz_2(control, data);

	GetClientRect(control, &rect);
	x1	= data->line1.to.x - 1;
	x2	= x1 - dx;
	y	= VERTICAL_MID(rect);
	y1	= y + dy;
	y2	= y - dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x2;
	data->head.list[0].y	= y;
	data->head.list[1].x	= x2;
	data->head.list[1].y	= y1;
	data->head.list[2].x	= x1;
	data->head.list[2].y	= y;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x2;
	data->head.list[4].y	= y;

	data->line1.to.x		-= dx;
	data->line2.to.x		-= dx;
	data->text.right		= data->line1.to.x;
}



/*****************************************************************************
*
*	Function:	_config_arrow_top_1
*
*	Purpose:
*
*		Configure the arrow for a single horizontal line pointing up.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_top_1(HWND control, arrow_t* data, int dx, int dy)
{
	int	x;
	int	x1;
	int	x2;
	int	y1;
	int	y2;

	_config_vert_1(control, data);

	x	= data->line1.from.x;
	x1	= x + dx;
	x2	= x - dx;
	y1	= data->line1.from.y;
	y2	= y1 + dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x;
	data->head.list[0].y	= y2;
	data->head.list[1].x	= x1;
	data->head.list[1].y	= y2;
	data->head.list[2].x	= x;
	data->head.list[2].y	= y1;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x;
	data->head.list[4].y	= y2;

	data->line1.from.y		+= dy;
	data->text.top			= data->line1.from.y;
}



/*****************************************************************************
*
*	Function:	_config_arrow_top_2
*
*	Purpose:
*
*		Configure the arrow for a pair of horizontal lines pointing up.
*
*	Arguments:
*
*		control		The control handle.
*
*		data		The control specific data we track.
*
*		dx			The length of the arrow head.
*
*		dy			Half the width of the arrow head.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _config_arrow_top_2(HWND control, arrow_t* data, int dx, int dy)
{
	RECT	rect;
	int		x;
	int		x1;
	int		x2;
	int		y1;
	int		y2;

	_config_vert_2(control, data);

	GetClientRect(control, &rect);
	x	= HORIZONTAL_MID(rect);
	x1	= x + dx;
	x2	= x - dx;
	y1	= data->line1.from.y;
	y2	= y1 + dy;

	data->head.enable		= TRUE;
	data->head.list[0].x	= x;
	data->head.list[0].y	= y2;
	data->head.list[1].x	= x1;
	data->head.list[1].y	= y2;
	data->head.list[2].x	= x;
	data->head.list[2].y	= y1;
	data->head.list[3].x	= x2;
	data->head.list[3].y	= y2;
	data->head.list[4].x	= x;
	data->head.list[4].y	= y2;

	data->line1.from.y		+= dy;
	data->line2.from.y		+= dy;
	data->text.top			= data->line1.from.y;
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		control		The control handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND control)
{
	arrow_t*	data	= (arrow_t*) GET_WINDOW_USER_DATA(control);

	if (data)
	{
		SET_WINDOW_USER_DATA(control, 0);
		free(data);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_enable
*
*	Purpose:
*
*		Process WM_ENABLE messages.
*
*	Arguments:
*
*		control		The control handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_enable(HWND control, WPARAM wParam)
{
	arrow_t*	data	= (arrow_t*) GET_WINDOW_USER_DATA(control);
	int			enabled	= wParam ? TRUE : FALSE;

	if (enabled)
	{
		data->head.brush		= data->device->paint.btntext.brush;
		data->head.pen			= data->device->paint.btntext.pen;
		data->line1.pen			= data->device->paint.btntext.pen;
		data->line2.pen			= data->device->paint.btntext.pen;
		data->line3.pen			= data->device->paint.btntext.pen;
		data->line4.pen			= data->device->paint.btntext.pen;
		data->line5.pen			= data->device->paint.btntext.pen;
		data->line6.pen			= data->device->paint.btntext.pen;
		data->rect.brush		= data->device->paint.btntext.brush;
		data->rect.pen			= data->device->paint.btntext.pen;
		data->text1.color_fg	= data->device->paint.btntext.color;
		data->text2.color_fg	= data->device->paint.btntext.color;
		InvalidateRect(control, NULL, TRUE);
	}
	else
	{
		data->head.brush		= data->device->paint.btnface.brush;
		data->head.pen			= data->device->paint.btnface.pen;
		data->line1.pen			= data->device->paint.btnface.pen;
		data->line2.pen			= data->device->paint.btnface.pen;
		data->line3.pen			= data->device->paint.btnface.pen;
		data->line4.pen			= data->device->paint.btnface.pen;
		data->line5.pen			= data->device->paint.btnface.pen;
		data->line6.pen			= data->device->paint.btnface.pen;
		data->rect.brush		= data->device->paint.btnface.brush;
		data->rect.pen			= data->device->paint.btnface.pen;
		data->text1.color_fg	= data->device->paint.btnface.color;
		data->text2.color_fg	= data->device->paint.btnface.color;
		InvalidateRect(control, NULL, TRUE);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_paint
*
*	Purpose:
*
*		Process WM_PAINT messages.
*
*	Arguments:
*
*		control		The control handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_paint(HWND control)
{
	arrow_t*	data	= (arrow_t*) GET_WINDOW_USER_DATA(control);
	HDC			dc;
	PAINTSTRUCT	ps;

	dc	= BeginPaint(control, &ps);
	_rect_paint(dc, &data->background);
	_line_paint(dc, &data->line1);
	_line_paint(dc, &data->line2);
	_line_paint(dc, &data->line3);
	_line_paint(dc, &data->line4);
	_line_paint(dc, &data->line5);
	_line_paint(dc, &data->line6);
	_head_paint(dc, &data->head);
	_rect_paint(dc, &data->rect);
	_text_paint(data, dc, &data->text1);
	_text_paint(data, dc, &data->text2);
	EndPaint(control, &ps);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_settext
*
*	Purpose:
*
*		Process WM_SETTEXT messages.
*
*	Arguments:
*
*		control		The control handle.
*
*		src			The text to apply to the control. We severly limit the
*					length.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_settext(HWND control, const char* src)
{
	arrow_t*	data	= (arrow_t*) GET_WINDOW_USER_DATA(control);
	char*		psz		= src ? strchr(src, ',') : NULL;

	if (data == NULL)
	{
		// No data to work with.
	}
	else if ((src == NULL) || (src[0] == 0))
	{
		// No text to display.
		data->text1.aligned	= FALSE;
		data->text1.enable	= FALSE;
		data->text2.aligned	= FALSE;
		data->text2.enable	= FALSE;
		InvalidateRect(control, NULL, TRUE);
	}
	else if (psz == NULL)
	{
		// Display a single line of text.
		data->text1.aligned	= FALSE;
		data->text1.enable	= TRUE;
		data->text2.aligned	= FALSE;
		data->text2.enable	= FALSE;
		strncpy(data->text1.text, src, sizeof(data->text1.text));
		data->text1.text[sizeof(data->text1.text) - 1]	= 0;
		_text_align(data);
		InvalidateRect(control, NULL, TRUE);
	}
	else
	{
		// Display two lines of text.
		data->text1.aligned	= FALSE;
		data->text1.enable	= TRUE;
		data->text2.aligned	= FALSE;
		data->text2.enable	= TRUE;
		strncpy(data->text2.text, psz + 1, sizeof(data->text2.text));
		data->text2.text[sizeof(data->text2.text) - 1]	= 0;
		strncpy(data->text1.text, src, sizeof(data->text1.text));
		data->text1.text[sizeof(data->text1.text) - 1]	= 0;
		psz	= strchr(data->text1.text, ',');

		if (psz)
			psz[0]	= 0;

		_text_align(data);
		InvalidateRect(control, NULL, TRUE);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_control_proc
*
*	Purpose:
*
*		Processes messages for the control.
*
*	Arguments:
*
*		control		The control handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _control_proc(	HWND	control,
										UINT	msg,
										WPARAM	wParam,
										LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(control);
			break;

		case WM_ENABLE:

			ret	= _wm_enable(control, wParam);
			break;

		case WM_PAINT:

			ret	= _wm_paint(control);
			break;

		case WM_SETTEXT:

			ret	= _wm_settext(control, (char*) lParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	arrow_type_set
*
*	Purpose:
*
*		Configure the arrow's display type.
*
*	Arguments:
*
*		dlg			The parent dialog handle.
*
*		id			The control id.
*
*		type		The flags that govern the arrow's appearance.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void arrow_type_set(HWND dlg, int id, arrow_type_t type)
{
	HWND		control	= GetDlgItem(dlg, id);
	arrow_t*	data	= (arrow_t*) GET_WINDOW_USER_DATA(control);
	int			dx		= (GetSystemMetrics(SM_CXSMICON) * 3) / 4;
	int			dy		= (GetSystemMetrics(SM_CYSMICON) * 3) / 4;

	switch (type)
	{
		default:
		case ARROW_NONE:

			_config_none(control, data);
			_text_align(data);
			break;

		case ARROW_BOTTOM:

			_config_arrow_bottom_1(control, data, (dx * 9) / 26, dy);
			_text_align(data);
			break;

		case ARROW_BOTTOM_2:

			_config_arrow_bottom_2(control, data, (dx * 9) / 26, dy);
			_text_align(data);
			break;

		case ARROW_HORZ:

			_config_horz_1(control, data);
			_text_align(data);
			break;

		case ARROW_HORZ_2:

			_config_horz_2(control, data);
			_text_align(data);
			break;

		case ARROW_LEFT:

			_config_arrow_left_1(control, data, dx, (dy * 9) / 26);
			_text_align(data);
			break;

		case ARROW_LEFT_2:

			_config_arrow_left_2(control, data, dx, (dy * 9) / 21);
			_text_align(data);
			break;

		case ARROW_RECT_SOLID:

			_config_rect_solid(control, data);
			_text_align(data);
			break;

		case ARROW_RIGHT:

			_config_arrow_right_1(control, data, dx, (dy * 9) / 26);
			_text_align(data);
			break;

		case ARROW_RIGHT_2:

			_config_arrow_right_2(control, data, dx, (dy * 9) / 21);
			_text_align(data);
			break;

		case ARROW_TOP:

			_config_arrow_top_1(control, data, (dx * 9) / 26, dy);
			_text_align(data);
			break;

		case ARROW_TOP_2:

			_config_arrow_top_2(control, data, (dx * 9) / 21, dy);
			_text_align(data);
			break;

		case ARROW_LR_2:

			_config_lower_right_2(control, data);
			_text_align(data);
			break;

		case ARROW_UR_2:

			_config_upper_right_2(control, data);
			_text_align(data);
			break;

		case ARROW_TR_2:

			_config_tee_right_2(control, data);
			_text_align(data);
			break;

		case ARROW_TRLL_2:

			_config_tee_right_lower_left_2(control, data);
			_text_align(data);
			break;

		case ARROW_TRUL_2:

			_config_tee_right_upper_left_2(control, data);
			_text_align(data);
			break;

		case ARROW_VERT:

			_config_vert_1(control, data);
			_text_align(data);
			break;

		case ARROW_VERT_2:

			_config_vert_2(control, data);
			_text_align(data);
			break;
	}

	InvalidateRect(control, NULL, TRUE);
}



/*****************************************************************************
*
*	Function:	arrow_init
*
*	Purpose:
*
*		Initialize a control to operate as a simple arrow.
*
*	Arguments:
*
*		dlg			The parent dialog handle.
*
*		device		The device data to use.
*
*		id			The control id.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void arrow_init(HWND dlg, device_t* device, int id)
{
	HWND		control;
	arrow_t*	data;
	HDC			dc;
	HFONT		font;
	TEXTMETRIC	tm;

	for (;;)	// A convenience loop.
	{
		control	= GetDlgItem(dlg, id);

		if (control == NULL)
			break;

		data	= (arrow_t*) malloc(sizeof(arrow_t));

		if (data == NULL)
			break;

		dc		= GetDC(dlg);
		font	= SelectObject(dc, device->font.defalt);
		GetTextMetrics(dc, &tm);
		SelectObject(dc, font);
		ReleaseDC(dlg, dc);

		memset(data, 0, sizeof(arrow_t));
		data->device	= device;
		data->background.brush	= data->device->paint.btnhighlt.brush;
		data->background.enable	= TRUE;
		data->background.pen	= data->device->paint.btnhighlt.pen;
		GetClientRect(control, &data->background.rect);
		data->head.brush		= data->device->paint.btntext.brush;
		data->head.pen			= data->device->paint.btntext.pen;
		data->line1.pen			= data->device->paint.btntext.pen;
		data->line2.pen			= data->device->paint.btntext.pen;
		data->rect.brush		= data->device->paint.btntext.brush;
		data->rect.pen			= data->device->paint.btntext.pen;
		data->text1.font		= device->font.defalt;
		data->text2.font		= device->font.defalt;
		data->tmAscent			= tm.tmAscent;
		data->tmDescent			= tm.tmDescent;
		SET_WINDOW_WND_PROC(control, _control_proc);
		SET_WINDOW_USER_DATA(control, data);
		break;
	}
}


