// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/c_code.c $
// $Rev: 3630 $
// $Date: 2009-10-12 10:21:12 -0500 (Mon, 12 Oct 2009) $

/*****************************************************************************
*
*	Implement services for generating sample C code.
*
*****************************************************************************/

// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/c_code.c $
// $Rev: 3630 $
// $Date: 2009-10-12 10:21:12 -0500 (Mon, 12 Oct 2009) $

#include <windows.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>

#include "c_code.h"
#include "align.h"
#include "cable.h"
#include "copy.h"
#include "fifo.h"
#include "gui.h"
#include "io.h"
#include "misc.h"
#include "resource.h"
#include "rx.h"
#include "tx.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"c code"

#define	_BUF_ENTER_PREFIX	"\n"											\
							"U32 hpdi32_sample_"

#define	_BUF_ENTER_SUFFIX	"_api_buffer"									\
							"(void* handle, U32* size, void** buffer)\n"	\
							"{\n"											\
							"    void*   api_buf_ptr     = NULL;\n"			\
							"    U32     api_buf_size    = 0;\n"			\
							"    U32     stat;\n"							\
							"    U32     status          = GSC_SUCCESS;\n"

#define	_BUF_EXIT			"\n"											\
							"    buffer[0]   = api_buf_ptr;\n"		\
							"    size[0]     = api_buf_size;\n"				\
							"    return(status);\n"							\
							"}\n"



/* typedefs *****************************************************************/

typedef struct
{
	device_t*	device;
	HWND		dlg;
	U32			status;
	char		status_global[1024 + _MAX_PATH];
	char		status_local[1024];
} code_t;

typedef struct
{
	const char*			preface;
	const parm_list_t*	list;
} group_t;

typedef struct
{
	const char*		enter;
	const group_t*	list;
	size_t			size;
	const char*		exit;
} function_t;



/* prototypes	*************************************************************/

static	LRESULT	_wm_size(HWND dlg);



/* variables	*************************************************************/

static const group_t	_sample_config_parms[]	=
{
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Miscellaneous Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&misc_parm_list
	},
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Cable Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&cable_parm_list
	},
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Tranmit I/O Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&tx_io_parm_list
	},
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Tranmit FIFO Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&fifo_tx_parm_list
	},
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Tranmitter Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&tx_parm_list
	},
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Receive I/O Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&rx_io_parm_list
	},
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Receive FIFO Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&fifo_rx_parm_list
	},
	{
		/* preface	*/	"\n"
						"    /*\n"
						"    *   Configure the Receiver Paramaters.\n"
						"    */\n"
						"\n",
		/* list		*/	&rx_parm_list
	}
};

static const function_t	_sample_config	=
{
	/* enter	*/	"\n"
					"U32 hpdi32_sample_config(void* handle)\n"
					"{\n"
					"    U32 stat;\n"
					"    U32 status  = GSC_SUCCESS;\n",
	/* list		*/	_sample_config_parms,
	/* size		*/	ARRAY_ELEMENTS(_sample_config_parms),
	/* exit		*/	"\n"
					"    return(status);\n"
					"}\n"
};

static const group_t	_sample_tx_buffer_parms[]	=
{
	{
		/* preface	*/	"",
		/* list		*/	&tx_io_parm_api_buffer_list
	}
};

static const function_t	_sample_tx_buffer	=
{
	/* enter	*/	_BUF_ENTER_PREFIX "tx" _BUF_ENTER_SUFFIX,
	/* list		*/	_sample_tx_buffer_parms,
	/* size		*/	ARRAY_ELEMENTS(_sample_tx_buffer_parms),
	/* exit		*/	_BUF_EXIT
};

static const group_t	_sample_rx_buffer_parms[]	=
{
	{
		/* preface	*/	"",
		/* list		*/	&rx_io_parm_api_buffer_list
	}
};

static const function_t	_sample_rx_buffer	=
{
	/* enter	*/	_BUF_ENTER_PREFIX "rx" _BUF_ENTER_SUFFIX,
	/* list		*/	_sample_rx_buffer_parms,
	/* size		*/	ARRAY_ELEMENTS(_sample_rx_buffer_parms),
	/* exit		*/	_BUF_EXIT
};



/*****************************************************************************
*
*	Function:	_show_window_status
*
*	Purpose:
*
*		Update the window to show the current status;
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		msg		The message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _show_window_status(HWND dlg, code_t* data)
{
	if (data->status != GSC_SUCCESS)
	{
		error_status(dlg, IDC_C_CODE_STATUS, data->status);
	}
	else if (data->status_global[0])
	{
		SetDlgItemText(dlg, IDC_C_CODE_STATUS, data->status_global);
	}
	else if (data->status_local[0])
	{
		data->status_global[0]	= 0;
		SetDlgItemText(dlg, IDC_C_CODE_STATUS, data->status_local);
	}
	else
	{
		SetDlgItemText(dlg, IDC_C_CODE_STATUS, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_add_line
*
*	Purpose:
*
*		Add a line of text to the list box.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		text		The text of the line to add.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _add_line(HWND dlg, const char* text)
{
	HWND	control		= GetDlgItem(dlg, IDC_C_CODE_LIST);

	SendMessage(control, LB_ADDSTRING, 0, (LPARAM) text);
}



/*****************************************************************************
*
*	Function:	_add_lines
*
*	Purpose:
*
*		Add a group of text lines to the list box. Each seperate line is
*		seperated by a '\n' character.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		text		The text of the line to add.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _add_lines(HWND dlg, const char* text)
{
	char	buf[2048];
	char*	psz;

	strcpy(buf, text);
	text	= buf;

	for(;;)
	{
		psz	= strchr(text, '\n');

		if (psz)
		{
			psz[0]	= 0;
			_add_line(dlg, text);
			text	= psz + 1;

			if (text[0] == 0)
				break;
		}
		else
		{
			_add_line(dlg, text);
			break;
		}
	}
}



/*****************************************************************************
*
*	Function:	_add_status_stat
*
*	Purpose:
*
*		Add a line folding a new status value into the existing status value.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _add_status_stat(HWND dlg)
{
	_add_line(dlg, "    status  = (status == GSC_SUCCESS) ? stat : status;");
}



/*****************************************************************************
*
*	Function:	_process_value_list
*
*	Purpose:
*
*		Compare the value to a value list and generate an appropriate HPDI32
*		macro code for applying that value.
*
*	Arguments:
*
*		value		The actual numeric value to apply.
*
*		values		The value set to evaluate.
*
*		set			The text pertaining to the "_SET" portion of the macro.
*
*		arg			The text pertaining to the ", argument" portion of the
*					macro.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static void _process_value_list(
	U32				value,
	const value_t*	values,
	char*			set,
	char*			arg)
{
	int					i;
	const value_list_t*	list	= values->u.list;

	for (i = 0; i < (int) list->size; i++)
	{
		if (value == list->list[i].value)
		{
			if (list->list[i].suffix[0])
			{
				strcpy(set, list->list[i].suffix);
				arg[0]	= 0;
			}
			else if (list->list[i].value_text[0])
			{
				strcpy(set, "_SET");
				strcpy(arg, ", ");
				strcat(arg, list->list[i].value_text);
			}
			else
			{
				strcpy(set, "_SET");
				strcpy(arg, ", ");
				sprintf(arg, ", %lu", (long) value);
			}

			break;
		}
	}

	if (i >= (int) list->size)
	{
		strcpy(set, "_SET");
		strcpy(arg, ", ");
		sprintf(arg, ", 0x%lX", (long) value);
	}
}



/*****************************************************************************
*
*	Function:	_process_value_range
*
*	Purpose:
*
*		Generate an appropriate HPDI32 macro code for applying the given value.
*
*	Arguments:
*
*		value		The actual numeric value to apply.
*
*		set			The text pertaining to the "_SET" portion of the macro.
*
*		arg			The text pertaining to the ", argument" portion of the
*					macro.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static void _process_value_range(U32 value, char* set, char* arg)
{
	strcpy(set, "_SET");
	strcpy(arg, ", ");
	sprintf(arg, ", 0x%lX", (long) value);
}



/*****************************************************************************
*
*	Function:	_process_value
*
*	Purpose:
*
*		Compare the value to the listed values and generate an appropriate
*		HPDI32 macro for applying that value.
*
*	Arguments:
*
*		value		The actual numeric value to apply.
*
*		values		The value set to evaluate.
*
*		set			The text pertaining to the "_SET" portion of the macro.
*
*		arg			The text pertaining to the ", argument" portion of the
*					macro.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static void _process_value(
	U32				value,
	const value_t*	values,
	char*			set,
	char*			arg)
{
	if (values->type == VALUE_TYPE_LIST)
		_process_value_list(value, values, set, arg);
	else if (values->type == VALUE_TYPE_RANGE)
		_process_value_range(value, set, arg);
}



/*****************************************************************************
*
*	Function:	_format_parm_entry_set
*
*	Purpose:
*
*		Generate the sample code for the given parameter, in which the form
*		of the code is to assign a parameter a value.
*
*	Arguments:
*
*		device		This is the device to access.
*
*		entry		The parameter to add.
*
*		value		The value to apply.
*
*		buf			The statement is recorded here. This is assumed to
*					plenty large enough.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _format_parm_entry_set(
	device_t*			device,
	const parm_entry_t*	entry,
	U32					value,
	char*				buf)
{
	char	arg[128]	= "";
	char	set[128]	= "";

	_process_value(value, entry->values, set, arg);
	buf[0]	= 0;
	strcat(buf, entry->parm_text);
	strcat(buf, "_");
	strcat(buf, entry->suffix);
	strcat(buf, set);
	strcat(buf, "(handle");
	strcat(buf, arg);
	strcat(buf, entry->set_arg);
	strcat(buf, ");");
}



/*****************************************************************************
*
*	Function:	_format_parm_entry_get
*
*	Purpose:
*
*		Generate sample code for retrievel of a parameter.
*
*	Arguments:
*
*		entry		The parameters to process.
*
*		buf			The statement is recorded here. This is presumed to be
*					plenty larg enough.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _format_parm_entry_get(const parm_entry_t* entry, char* buf)
{
	buf[0]	= 0;
	strcat(buf, entry->parm_text);
	strcat(buf, "_");
	strcat(buf, entry->suffix);
	strcat(buf, "_GET");
	strcat(buf, "(handle");
	strcat(buf, entry->get_arg);
	strcat(buf, ");");
}



/*****************************************************************************
*
*	Function:	_add_parm_list
*
*	Purpose:
*
*		Generate the sample code for the given parameter list, in which the
*		form of the code is to assign a parameter a value.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		list		The parameter list to add.
*
*		apply		Apply the current settings?
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _add_parm_list(HWND dlg, code_t* data, const parm_list_t* list, int apply)
{
	char				buf[1024];
	const parm_entry_t*	entry;
	int					i;
	U32					stat;
	U32					status	= GSC_SUCCESS;
	U32					value;

	for (i = 0; i < (int) list->size; i++)
	{
		entry	= list->list[i];

		stat	= hpdi32_config(data->device->handle,
								entry->parm,
								list->which | entry->which,
								GSC_NO_CHANGE,
								&value);
		status	= (status == GSC_SUCCESS) ? stat : status;

		strcpy(buf, "    stat    = ");
		c_code_format_entry(	data->device,
								list,
								i,
								apply,
								value,
								buf + strlen(buf));

		_add_line(dlg, buf);
		_add_status_stat(dlg);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_add_group
*
*	Purpose:
*
*		Generate the sample code for the given parameter group, in which the
*		form of the code is to assign a parameter a value.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		group		The parameter group to add.
*
*		apply		Apply the current settings?
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _add_group(HWND dlg, code_t* data, const group_t* group, int apply)
{
	U32	status;

	_add_lines(dlg, group->preface);
	status	= _add_parm_list(dlg, data, group->list, apply);
	return(status);
}



/*****************************************************************************
*
*	Function:	_add_function
*
*	Purpose:
*
*		Generate the sample code for the configuration function.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		func		The function definition to add.
*
*		apply		Apply the current settings?
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _add_function(HWND dlg, code_t* data, const function_t* func, int apply)
{
	int	i;
	U32	stat;
	U32	status	= GSC_SUCCESS;

	_add_lines(dlg, func->enter);

	for (i = 0; i < (int) func->size; i++)
	{
		stat	= _add_group(dlg, data, &func->list[i], apply);
		status	= (status == GSC_SUCCESS) ? stat : status;
	}

	_add_lines(dlg, func->exit);
	return(status);
}



/*****************************************************************************
*
*	Function:	_generate_c_code
*
*	Purpose:
*
*		Generate the sample code per the current device state.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _generate_c_code(HWND dlg, code_t* data)
{
	char		buf[128];
	HWND		control;
	LRESULT		count;
	HDC			dc;
	int			i;
	int			j;
	char*		psz;
	WPARAM		sel;
	U32			stat;
	U32			status	= GSC_SUCCESS;
	struct tm*	stm;
	WPARAM		top;
	TEXTMETRIC	tm;
	time_t		tt;
	int			width;

	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	strcpy(data->status_local, "Generated C source code: ");
	strcat(data->status_local, psz);
	sprintf(buf, "*   Generated: %s", psz);
	psz	= buf + strlen(buf) - 1;

	for (; iscntrl(psz[0]);)
	{
		psz[0]	= 0;
		psz--;
	}

	control	= GetDlgItem(dlg, IDC_C_CODE_LIST);
	SendMessage(control, WM_SETREDRAW, FALSE, 0);
	sel		= (WPARAM) SendMessage(control, LB_GETCURSEL, 0, 0);
	top		= (WPARAM) SendMessage(control, LB_GETTOPINDEX, 0, 0);
	SendMessage(control, LB_RESETCONTENT, 0, 0);

	_add_line(dlg, "/*");
	_add_line(dlg, "*   Sample code generated by Visual HPDI32 (" VERSION_STRING ")");
	_add_line(dlg, buf);
	_add_line(dlg, "*/");
	_add_line(dlg, "");
	_add_line(dlg, "#include \"hpdi32.h\"");
	_add_line(dlg, "#include \"hpdi32_sample.h\"");
	_add_line(dlg, "");

	stat	= _add_function(dlg, data, &_sample_config, TRUE);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _add_function(dlg, data, &_sample_tx_buffer, FALSE);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _add_function(dlg, data, &_sample_rx_buffer, FALSE);
	status	= (status == GSC_SUCCESS) ? stat : status;

	SendMessage(control, LB_SETCURSEL, sel, 0);
	SendMessage(control, LB_SETTOPINDEX, top, 0);
	SendMessage(control, WM_SETREDRAW, TRUE, 0);

	count	= SendMessage(control, LB_GETCOUNT, 0, 0);
	width	= 0;
	dc		= GetDC(control);
	GetTextMetrics(dc, &tm);
	ReleaseDC(control, dc);

	for (i = 0; i < count; i++)
	{
		SendMessage(control, LB_GETTEXT, (WPARAM) i, (LPARAM) buf);
		j	= (int) strlen(buf);

		if (width < j)
			width	= j;
	}

	width	= (width + 2) * tm.tmAveCharWidth;
	SendMessage(control, LB_SETHORIZONTALEXTENT, (WPARAM) width, 0);

	if (status != GSC_SUCCESS)
		data->status	= status;
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window's controls per the state of the device
*		in use, if any.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device structure to access.
*
*		msg		Any information message to appear in the status box.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	code_t*	data;

	data					= (code_t*) GET_WINDOW_USER_DATA(dlg);
	data->status			= GSC_SUCCESS;
	data->status_global[0]	= 0;

	set_win_title(dlg, data->device, FALSE, TRUE, "C Source Code");

	if (msg)
		strncpy(data->status_global, msg, sizeof(data->status_global));

	if (device->handle)
	{
		ctrl_menu_set(dlg, IDC_SAVE, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_C_CODE_LIST, CTRL_ENABLE);
		_generate_c_code(dlg, data);
	}
	else
	{
		ctrl_menu_set(dlg, IDC_SAVE, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
		ctrl_menu_set(dlg, IDC_C_CODE_LIST, CTRL_DISABLE | CTRL_LB_RESET);
		data->status_local[0]	= 0;
	}

	_show_window_status(dlg, data);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id					flags						*/
		{ -1,					COPY_EOL | COPY_UNDERLINE	},

		{ IDC_C_CODE_LIST,		COPY_EOL | COPY_LIST_BOX	},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_SAVE,				COPY_EOL					},
		{ IDCANCEL,				COPY_EOL					},

		{ 0,					COPY_UNDERLINE				},
		{ 0,					COPY_EOL					},

		{ IDC_C_CODE_STATUS,	COPY_EOL					}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_save
*
*	Purpose:
*
*		Prompt for a file name and save the generated C code.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_save(HWND dlg)
{
	TCHAR			buf[1024];
	HWND			control;
	LRESULT			count;
	code_t*			data;
	char			dir[_MAX_PATH];
	FILE*			file;
	int				j;
	size_t			len;
	int				i;
	TCHAR			name[_MAX_PATH];
	OPENFILENAME	ofn;

	data					= (code_t*) GET_WINDOW_USER_DATA(dlg);
	data->status			= GSC_SUCCESS;
	data->status_local[0]	= 0;
	strcpy(name, "hpdi32_sample.c");
	GetCurrentDirectory(sizeof(dir), dir);

	// Initialize OPENFILENAME
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= dlg;
	ofn.lpstrFilter		= "C Source Files (*.c)\0*.c\0"
							"All (*.*)\0*.*\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFile		= name;
	ofn.nMaxFile		= sizeof(name);
	ofn.lpstrInitialDir	= dir;
	ofn.lpstrTitle		= "Save C Source Code";
	ofn.Flags			= OFN_PATHMUSTEXIST
						| OFN_OVERWRITEPROMPT
						| OFN_HIDEREADONLY;
	i					= GetSaveFileName(&ofn);

	for (;i;)
	{
		buf[0]	= 0;
		file	= fopen(name, "wb");

		if (file == NULL)
		{
			sprintf(data->status_local, "Unable to create '%s'.", name);
			break;
		}

		control	= GetDlgItem(dlg, IDC_C_CODE_LIST);
		count	= SendMessage(control, LB_GETCOUNT, 0, 0);
		j		= 0;
		len		= 0;

		for (i = 0; i < count; i++)
		{
			SendMessage(control, LB_GETTEXT, (WPARAM) i, (LPARAM) buf);
			strcat(buf, "\r\n");
			len	= strlen(buf);
			j	= (int) fwrite(buf, 1, len, file);

			if (j != (int) len)
			{
				sprintf(data->status_local, "Write failure to '%s'.", name);
				break;
			}
		}

		fclose(file);

		if (j == (int) len)
			sprintf(data->status_local, "Saved C source code to '%s'.", name);

		break;
	}

	_show_window_status(dlg, data);
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	code_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (code_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	int		id	= LOWORD(wParam);
	LRESULT	ret	= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_SAVE:

			_idc_save(dlg);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_getminmaxinfo
*
*	Purpose:
*
*		Process a WM_GETMINMAXINFO message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The messages's long size parameter.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_getminmaxinfo(HWND dlg, LPARAM lParam)
{
	HWND		control;
	MINMAXINFO*	mmi		= (MINMAXINFO*) lParam;
	RECT		rect;

	// Width
	control	= GetDlgItem(dlg, IDC_C_CODE_LIST);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	= rect.left;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.x	-= rect.left;
	mmi->ptMinTrackSize.x	*= 2;

	control	= GetDlgItem(dlg, IDCANCEL);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	+= rect.right;

	control	= GetDlgItem(dlg, IDC_SAVE);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.x	-= rect.left;

	// Height
	control	= GetDlgItem(dlg, IDC_C_CODE_STATUS);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	= rect.bottom;

	control	= GetDlgItem(dlg, IDC_C_CODE_LIST);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	-= rect.bottom;
	mmi->ptMinTrackSize.y	+= rect.top;

	GetWindowRect(dlg, &rect);
	mmi->ptMinTrackSize.y	-= rect.top;

	control	= GetDlgItem(dlg, IDC_SAVE);
	GetWindowRect(control, &rect);
	mmi->ptMinTrackSize.y	+= 3 * (rect.bottom - rect.top);

	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	HWND	control;
	code_t*	data	= (code_t*) lParam;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control	= GetDlgItem(dlg, IDC_C_CODE_LIST);
	SendMessage(control, WM_SETFONT, (WPARAM) data->device->font.fixed, 0);

	win_pos_restore(dlg, SECTION_NAME, TRUE);
	_wm_size(dlg);
	main_refresh_add(dlg, _refresh_window);
	_refresh_window(dlg, data->device, NULL);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_size
*
*	Purpose:
*
*		Process a WM_SIZE message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_size(HWND dlg)
{
	code_t*			data	= (code_t*) GET_WINDOW_USER_DATA(dlg);

	static align_t	list[]	=
	{
		// The outter frame.
		{// 0
		/* id		*/	IDC_C_CODE_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-1
		},
		{// 1
		/* id		*/	IDC_C_CODE_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_T,
		/* value	*/	0
		},
		{// 2
		/* id		*/	IDC_C_CODE_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R | ALIGN_B_B,
		/* value	*/	1
		},

		// The status text.
		{// 3
		/* id		*/	IDC_C_CODE_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 4
		/* id		*/	IDC_C_CODE_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},
		{// 5
		/* id		*/	IDC_C_CODE_STATUS,
		/* ref		*/	0,
		/* flags	*/	ALIGN_B_B | ALIGN_H,
		/* value	*/	- MARGIN
		},

		// The status frame.
		{// 6
		/* id		*/	IDC_C_CODE_FRAME_2,
		/* ref		*/	IDC_C_CODE_FRAME_1,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R | ALIGN_B_B,
		/* value	*/	0
		},
		{// 7
		/* id		*/	IDC_C_CODE_FRAME_2,
		/* ref		*/	IDC_C_CODE_STATUS,
		/* flags	*/	ALIGN_T_T,
		/* value	*/	- MARGIN
		},

		// The SAVE button.
		{// 8
		/* id		*/	IDC_SAVE,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_C | ALIGN_W,
		/* value	*/	-999	// see below
		},
		{// 9
		/* id		*/	IDC_SAVE,
		/* ref		*/	IDC_C_CODE_FRAME_2,
		/* flags	*/	ALIGN_B_T | ALIGN_H,
		/* value	*/	-999	// see below
		},

		// The CLOSE button.
		{// 10
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_SAVE,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 12
		/* id		*/	IDCANCEL,
		/* ref		*/	IDC_SAVE,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The list box.
		{// 13
		/* id		*/	IDC_C_CODE_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-999	// see below
		},
		{// 14
		/* id		*/	IDC_C_CODE_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_T,
		/* value	*/	-999	// see below
		},
		{// 15
		/* id		*/	IDC_C_CODE_LIST,
		/* ref		*/	0,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// see below
		},
		{// 16
		/* id		*/	IDC_C_CODE_LIST,
		/* ref		*/	IDC_SAVE,
		/* flags	*/	ALIGN_B_T,
		/* value	*/	-999	// see below
		},

		{ 0, 0, 0, 0 }
	};

	list[3].value	= data->device->margin;
	list[4].value	= - data->device->margin;
	list[8].value	= - data->device->margin / 2;
	list[9].value	= - data->device->margin;
	list[12].value	= data->device->margin;
	list[13].value	= data->device->margin;
	list[14].value	= 1 + data->device->margin;
	list[15].value	= - data->device->margin;
	list[16].value	= - data->device->margin;

	align_control_list(dlg, list);
	InvalidateRect(dlg, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_GETMINMAXINFO:

			ret	= _wm_getminmaxinfo(dlg, lParam);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SIZE:

			ret	= _wm_size(dlg);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_C_CODE, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_C_CODE, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	code_t	data;

	if (data.dlg)	/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.device	= device;
		data.dlg	= CreateDialogParam(device->instance,
										(LPCTSTR) IDD_C_CODE,
										dlg,
										(DLGPROC) _dlg_proc,
										(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	c_code_service
*
*	Purpose:
*
*		Implement the common service entry point for the C Code services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT c_code_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	c_code_format_entry
*
*	Purpose:
*
*		Generate the sample code for the given parameter list, in which the
*		form of the code is to assign a parameter a value.
*
*	Arguments:
*
*		device		The device to access.
*
*		list		The parameter list to access.
*
*		index		The list entry to access.
*
*		apply		Apply the current settings?
*
*		value		The value to apply.
*
*		buf			The statement is recorded here. This is presumed to be
*					plenty large enough.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void c_code_format_entry(
	device_t*			device,
	const parm_list_t*	list,
	int					index,
	int					apply,
	U32					value,
	char*				buf)
{
	parm_entry_t	entry;
	char			suffix[32];

	entry	= list->list[index][0];

	if (list->suffix)
	{
		// Apply overrides.
		strcpy(suffix, list->suffix);
		strcat(suffix, entry.suffix);
		entry.suffix	= suffix;
		entry.which		|= list->which;
	}

	if (apply)
		_format_parm_entry_set(device, &entry, value, buf);
	else
		_format_parm_entry_get(&entry, buf);
}


