// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/cable_clock.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the cable's Clock configuration option.
*
*****************************************************************************/

#include <windows.h>

#include "cable.h"
#include "align.h"
#include "arrow.h"
#include "resource.h"
#include "static.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	align_t list[]	=
	{
		// The Cable Tx Clock arrow.
		{// 0
		/* id		*/	IDC_CABLE_TX_CLOCK_ARROW,
		/* ref		*/	IDC_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 1
		/* id		*/	IDC_CABLE_TX_CLOCK_ARROW,
		/* ref		*/	IDC_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	MARGIN
		},
		{// 2
		/* id		*/	IDC_CABLE_TX_CLOCK_ARROW,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The Cable Tx Clock extension.
		{// 3
		/* id		*/	IDC_CABLE_TX_CLOCK_EXT,
		/* ref		*/	IDC_CABLE_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_R | ALIGN_T_C,
		/* value	*/	0
		},
		{// 4
		/* id		*/	IDC_CABLE_TX_CLOCK_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},
		{// 5
		/* id		*/	IDC_CABLE_TX_CLOCK_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_H,
		/* value	*/	1
		},

		// The Cable Rx Clock arrow.
		{// 6
		/* id		*/	IDC_CABLE_RX_CLOCK_ARROW,
		/* ref		*/	IDC_RX_CLOCK_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 7
		/* id		*/	IDC_CABLE_RX_CLOCK_ARROW,
		/* ref		*/	IDC_CABLE_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},

		// The Cable Rx Clock extension.
		{// 8
		/* id		*/	IDC_CABLE_RX_CLOCK_EXT,
		/* ref		*/	IDC_CABLE_TX_CLOCK_EXT,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 9
		/* id		*/	IDC_CABLE_RX_CLOCK_EXT,
		/* ref		*/	IDC_CABLE_RX_CLOCK_ARROW,
		/* flags	*/	ALIGN_T_C,
		/* value	*/	0
		},
		{// 10
		/* id		*/	IDC_CABLE_RX_CLOCK_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_H,
		/* value	*/	1
		},

		// The Cable Clock extension.
		{// 11
		/* id		*/	IDC_CABLE_CLOCK_EXT,
		/* ref		*/	IDC_CABLE_TX_CLOCK_EXT,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	0
		},
		{// 12
		/* id		*/	IDC_CABLE_CLOCK_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},
		{// 13
		/* id		*/	IDC_CABLE_CLOCK_EXT,
		/* ref		*/	IDC_TX_FV_ARROW,
		/* flags	*/	ALIGN_T_T,
		/* value	*/	0
		},
		{// 14
		/* id		*/	IDC_CABLE_CLOCK_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_H,
		/* value	*/	1
		},

		// The Cable Clock arrow.
		{// 15
		/* id		*/	IDC_CABLE_CLOCK_ARROW,
		/* ref		*/	IDC_CABLE_CLOCK_EXT,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	0
		},
		{// 16
		/* id		*/	IDC_CABLE_CLOCK_ARROW,
		/* ref		*/	IDC_TX_FV_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 17
		/* id		*/	IDC_CABLE_CLOCK_ARROW,
		/* ref		*/	IDC_CABLE_CLOCK_EXT,
		/* flags	*/	ALIGN_VC_T | ALIGN_H,
		/* value	*/	0
		},
		{// 18
		/* id		*/	IDC_CABLE_CLOCK_ARROW,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The Cable Clock control.
		{// 19
		/* id		*/	IDC_CABLE_CLOCK,
		/* ref		*/	IDC_CABLE_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_R | ALIGN_W | ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The Cable Tx Clock vertical bar.
		{// 20
		/* id		*/	IDC_CABLE_TX_CLOCK_VERT,
		/* ref		*/	IDC_CABLE_TX_CLOCK_EXT,
		/* flags	*/	ALIGN_L_R | ALIGN_T_T,
		/* value	*/	0
		},
		{// 21
		/* id		*/	IDC_CABLE_TX_CLOCK_VERT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	1
		},
		{// 22
		/* id		*/	IDC_CABLE_TX_CLOCK_VERT,
		/* ref		*/	IDC_CABLE_CLOCK_EXT,
		/* flags	*/	ALIGN_B_T,
		/* value	*/	-1
		},

		// The Cable Rx Clock arrow vertical bar.
		{// 23
		/* id		*/	IDC_CABLE_RX_CLOCK_VERT,
		/* ref		*/	IDC_CABLE_TX_CLOCK_VERT,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 24
		/* id		*/	IDC_CABLE_RX_CLOCK_VERT,
		/* ref		*/	IDC_CABLE_CLOCK_EXT,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	1
		},
		{// 25
		/* id		*/	IDC_CABLE_RX_CLOCK_VERT,
		/* ref		*/	IDC_CABLE_RX_CLOCK_EXT,
		/* flags	*/	ALIGN_B_B,
		/* value	*/	0
		},

		// The Cable Clock junction.
		{// 26
		/* id		*/	IDC_CABLE_CLOCK_JUNC,
		/* ref		*/	IDC_CABLE_CLOCK_EXT,
		/* flags	*/	ALIGN_L_L | ALIGN_T_T,
		/* value	*/	-1
		},
		{// 27
		/* id		*/	IDC_CABLE_CLOCK_JUNC,
		/* ref		*/	IDC_CABLE_CLOCK_JUNC,
		/* flags	*/	ALIGN_R_L | ALIGN_B_T,
		/* value	*/	3
		},

		{ 0, 0, 0, 0 }
	};

	list[2].value	= device->btn_height;
	list[4].value	= (device->btn_height * 15) / 2;
	list[12].value	= (device->btn_height * 3) / 2;
	list[18].value	= device->btn_height;

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_CABLE_CLOCK);

	arrow_init(dlg, device, IDC_CABLE_CLOCK_ARROW);
	arrow_type_set(dlg, IDC_CABLE_CLOCK_ARROW, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_CLOCK_EXT);
	arrow_type_set(dlg, IDC_CABLE_CLOCK_EXT, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_CLOCK_JUNC);
	arrow_type_set(dlg, IDC_CABLE_CLOCK_JUNC, ARROW_RECT_SOLID);

	arrow_init(dlg, device, IDC_CABLE_TX_CLOCK_ARROW);
	arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_TX_CLOCK_EXT);
	arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_EXT, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_TX_CLOCK_VERT);
	arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_VERT, ARROW_VERT);

	arrow_init(dlg, device, IDC_CABLE_RX_CLOCK_ARROW);
	arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_RX_CLOCK_EXT);
	arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_EXT, ARROW_HORZ);
	arrow_init(dlg, device, IDC_CABLE_RX_CLOCK_VERT);
	arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_VERT, ARROW_VERT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;
	U32		rx;
	U32		status;
	U32		tx;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// Cable Clock is inactive as the device is closed.
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_JUNC, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_VERT, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_VERT, CTRL_DISABLE);

			arrow_type_set(dlg, IDC_CABLE_CLOCK_ARROW, ARROW_HORZ);
			arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, ARROW_HORZ);
			arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, ARROW_HORZ);
			break;
		}

		// The device is open.
		status	= HPDI32_TX_ENABLE__GET(device->handle, &tx);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_RX_ENABLE__GET(device->handle, &rx);
		error_status(dlg, IDC_INFO_TEXT, status);

		if ((tx == HPDI32_TX_ENABLE_YES) && (rx == HPDI32_RX_ENABLE_YES))
		{
			// Cable Clock is Tx Clock (output) and Rx Clock.
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_JUNC, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_EXT, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_VERT, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_VERT, CTRL_ENABLE);

			arrow_type_set(dlg, IDC_CABLE_CLOCK_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, ARROW_LEFT);
			break;
		}

		if (tx == HPDI32_TX_ENABLE_YES)
		{
			// Cable Clock is Tx Clock (output).
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_VERT, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_VERT, CTRL_DISABLE);

			arrow_type_set(dlg, IDC_CABLE_CLOCK_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, ARROW_RIGHT);
			arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, ARROW_HORZ);
			break;
		}

		// The signal is not driven by the transmitter.

		if (rx == HPDI32_RX_ENABLE_YES)
		{
			// Cable Clock is Rx Clock (input).
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_CLOCK_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_VERT, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_VERT, CTRL_ENABLE);

			arrow_type_set(dlg, IDC_CABLE_CLOCK_ARROW, ARROW_LEFT);
			arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, ARROW_HORZ);
			arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, ARROW_LEFT);
			break;
		}

		// Cable Clock is inactive at present.
		ctrl_menu_set(dlg, IDC_CABLE_CLOCK, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_CABLE_CLOCK_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_CLOCK_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_CLOCK_JUNC, CTRL_DISABLE);

		ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_TX_CLOCK_VERT, CTRL_DISABLE);

		ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_RX_CLOCK_VERT, CTRL_DISABLE);

		arrow_type_set(dlg, IDC_CABLE_CLOCK_ARROW, ARROW_HORZ);
		arrow_type_set(dlg, IDC_CABLE_TX_CLOCK_ARROW, ARROW_HORZ);
		arrow_type_set(dlg, IDC_CABLE_RX_CLOCK_ARROW, ARROW_HORZ);
		break;
	}

	control	= GetDlgItem(dlg, IDC_CABLE_CLOCK_ARROW);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	cable_clock_service
*
*	Purpose:
*
*		Implement the common service entry point for the Cable Clock services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT cable_clock_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


