// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/cable_data.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Cable Data option.
*
*****************************************************************************/

#include <windows.h>

#include "cable.h"
#include "align.h"
#include "arrow.h"
#include "resource.h"
#include "static.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	align_t list[]	=
	{
		// The Cable Tx Data arrow.
		{// 0
		/* id		*/	IDC_CABLE_TX_DATA_ARROW,
		/* ref		*/	IDC_TX_DF_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 1
		/* id		*/	IDC_CABLE_TX_DATA_ARROW,
		/* ref		*/	IDC_TX_DF_ARROW,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	MARGIN
		},
		{// 2
		/* id		*/	IDC_CABLE_TX_DATA_ARROW,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The Cable Tx Data extension.
		{// 3
		/* id		*/	IDC_CABLE_TX_DATA_EXT,
		/* ref		*/	IDC_CABLE_TX_DATA_ARROW,
		/* flags	*/	ALIGN_L_R | ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 4
		/* id		*/	IDC_CABLE_TX_DATA_EXT,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The Cable Tx Data junction.
		{// 5
		/* id		*/	IDC_CABLE_TX_DATA_JUNC,
		/* ref		*/	IDC_CABLE_TX_DATA_EXT,
		/* flags	*/	ALIGN_L_R | ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 6
		/* id		*/	IDC_CABLE_TX_DATA_JUNC,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The Cable Rx Data arrow.
		{// 7
		/* id		*/	IDC_CABLE_RX_DATA_ARROW,
		/* ref		*/	IDC_RX_DATA,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 8
		/* id		*/	IDC_CABLE_RX_DATA_ARROW,
		/* ref		*/	IDC_CABLE_TX_DATA_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},

		// The Cable Rx Data extension.
		{// 9
		/* id		*/	IDC_CABLE_RX_DATA_EXT,
		/* ref		*/	IDC_CABLE_RX_DATA_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 10
		/* id		*/	IDC_CABLE_RX_DATA_EXT,
		/* ref		*/	IDC_CABLE_TX_DATA_EXT,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},

		// The Cable Rx Data junction.
		{// 11
		/* id		*/	IDC_CABLE_RX_DATA_JUNC,
		/* ref		*/	IDC_CABLE_RX_DATA_EXT,
		/* flags	*/	ALIGN_L_R | ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 12
		/* id		*/	IDC_CABLE_RX_DATA_JUNC,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	-999	// see below
		},

		// The Cable Data control.
		{// 13
		/* id		*/	IDC_CABLE_DATA,
		/* ref		*/	IDC_CABLE_CLOCK,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 14
		/* id		*/	IDC_CABLE_DATA,
		/* ref		*/	IDC_CABLE_CLOCK,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},

		// The Cable Data arrow.
		{// 15
		/* id		*/	IDC_CABLE_DATA_ARROW,
		/* ref		*/	IDC_CABLE_CLOCK_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{// 16
		/* id		*/	IDC_CABLE_DATA_ARROW,
		/* ref		*/	IDC_CABLE_CLOCK_ARROW,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	0
		},

		// The Cable Data extension.
		{// 17
		/* id		*/	IDC_CABLE_DATA_EXT,
		/* ref		*/	IDC_CABLE_TX_DATA_JUNC,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	0
		},
		{// 18
		/* id		*/	IDC_CABLE_DATA_EXT,
		/* ref		*/	IDC_CABLE_DATA_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B | ALIGN_R_L,
		/* value	*/	0
		},

		// The Cable Data junction.
		{// 19
		/* id		*/	IDC_CABLE_DATA_JUNC,
		/* ref		*/	IDC_CABLE_TX_DATA_JUNC,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 20
		/* id		*/	IDC_CABLE_DATA_JUNC,
		/* ref		*/	IDC_CABLE_DATA_EXT,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},

		// The Cable Tx Data vertical bar.
		{// 21
		/* id		*/	IDC_CABLE_TX_DATA_VERT,
		/* ref		*/	IDC_CABLE_TX_DATA_JUNC,
		/* flags	*/	ALIGN_T_B | ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 22
		/* id		*/	IDC_CABLE_TX_DATA_VERT,
		/* ref		*/	IDC_CABLE_DATA_JUNC,
		/* flags	*/	ALIGN_B_T,
		/* value	*/	0
		},

		// The Cable Rx Data vertical bar.
		{// 23
		/* id		*/	IDC_CABLE_RX_DATA_VERT,
		/* ref		*/	IDC_CABLE_RX_DATA_JUNC,
		/* flags	*/	ALIGN_B_T | ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{// 24
		/* id		*/	IDC_CABLE_RX_DATA_VERT,
		/* ref		*/	IDC_CABLE_DATA_JUNC,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	0
		},

		{ 0, 0, 0, 0 }
	};

	list[2].value	= device->btn_height;
	list[4].value	= (device->btn_height * 12) / 2;
	list[6].value	= device->btn_height;
	list[12].value	= device->btn_height;

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_CABLE_DATA);

	arrow_init(dlg, device, IDC_CABLE_DATA_ARROW);
	arrow_type_set(dlg, IDC_CABLE_DATA_ARROW, ARROW_HORZ_2);
	arrow_init(dlg, device, IDC_CABLE_DATA_EXT);
	arrow_type_set(dlg, IDC_CABLE_DATA_EXT, ARROW_HORZ_2);
	arrow_init(dlg, device, IDC_CABLE_DATA_JUNC);
	arrow_type_set(dlg, IDC_CABLE_DATA_JUNC, ARROW_TR_2);

	arrow_init(dlg, device, IDC_CABLE_TX_DATA_ARROW);
	arrow_type_set(dlg, IDC_CABLE_TX_DATA_ARROW, ARROW_HORZ_2);
	arrow_init(dlg, device, IDC_CABLE_TX_DATA_EXT);
	arrow_type_set(dlg, IDC_CABLE_TX_DATA_EXT, ARROW_HORZ_2);
	arrow_init(dlg, device, IDC_CABLE_TX_DATA_JUNC);
	arrow_type_set(dlg, IDC_CABLE_TX_DATA_JUNC, ARROW_UR_2);
	arrow_init(dlg, device, IDC_CABLE_TX_DATA_VERT);
	arrow_type_set(dlg, IDC_CABLE_TX_DATA_VERT, ARROW_VERT_2);

	arrow_init(dlg, device, IDC_CABLE_RX_DATA_ARROW);
	arrow_type_set(dlg, IDC_CABLE_RX_DATA_ARROW, ARROW_HORZ_2);
	arrow_init(dlg, device, IDC_CABLE_RX_DATA_EXT);
	arrow_type_set(dlg, IDC_CABLE_RX_DATA_EXT, ARROW_HORZ_2);
	arrow_init(dlg, device, IDC_CABLE_RX_DATA_JUNC);
	arrow_type_set(dlg, IDC_CABLE_RX_DATA_JUNC, ARROW_LR_2);
	arrow_init(dlg, device, IDC_CABLE_RX_DATA_VERT);
	arrow_type_set(dlg, IDC_CABLE_RX_DATA_VERT, ARROW_VERT_2);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service for
*		the GUI window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;
	U32		rx;
	U32		status;
	U32		tx;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// Cable Data is inactive as the device is closed.
			ctrl_menu_set(dlg, IDC_CABLE_DATA, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_JUNC, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_JUNC, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_VERT, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_JUNC, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_VERT, CTRL_DISABLE);

			arrow_type_set(dlg, IDC_CABLE_DATA_ARROW, ARROW_HORZ_2);
			arrow_type_set(dlg, IDC_CABLE_DATA_JUNC, ARROW_TR_2);
			arrow_type_set(dlg, IDC_CABLE_TX_DATA_ARROW, ARROW_HORZ_2);
			arrow_type_set(dlg, IDC_CABLE_RX_DATA_ARROW, ARROW_HORZ_2);
			break;
		}

		// The device is open.
		status	= HPDI32_TX_ENABLE__GET(device->handle, &tx);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_RX_ENABLE__GET(device->handle, &rx);
		error_status(dlg, IDC_INFO_TEXT, status);

		if ((tx == HPDI32_TX_ENABLE_YES) && (rx == HPDI32_RX_ENABLE_YES))
		{
			// Cable Data is Tx Data (output) and Rx Data.
			ctrl_menu_set(dlg, IDC_CABLE_DATA, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_JUNC, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_VERT, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_JUNC, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_VERT, CTRL_ENABLE);

			arrow_type_set(dlg, IDC_CABLE_DATA_ARROW, ARROW_RIGHT_2);
			arrow_type_set(dlg, IDC_CABLE_DATA_JUNC, ARROW_TR_2);
			arrow_type_set(dlg, IDC_CABLE_TX_DATA_ARROW, ARROW_RIGHT_2);
			arrow_type_set(dlg, IDC_CABLE_RX_DATA_ARROW, ARROW_LEFT_2);
			break;
		}

		if (tx == HPDI32_TX_ENABLE_YES)
		{
			// Cable Data is Tx Data (output).
			ctrl_menu_set(dlg, IDC_CABLE_DATA, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_JUNC, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_VERT, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_JUNC, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_VERT, CTRL_DISABLE);

			arrow_type_set(dlg, IDC_CABLE_DATA_ARROW, ARROW_RIGHT_2);
			arrow_type_set(dlg, IDC_CABLE_DATA_JUNC, ARROW_TRLL_2);
			arrow_type_set(dlg, IDC_CABLE_TX_DATA_ARROW, ARROW_RIGHT_2);
			arrow_type_set(dlg, IDC_CABLE_RX_DATA_ARROW, ARROW_HORZ_2);
			break;
		}

		// The transmitter is disabled.

		if (rx == HPDI32_RX_ENABLE_YES)
		{
			// Cable Data is Rx Data (input).
			ctrl_menu_set(dlg, IDC_CABLE_DATA, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_DATA_JUNC, CTRL_ENABLE);

			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_ARROW, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_EXT, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_JUNC, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_VERT, CTRL_DISABLE);

			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_EXT, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_JUNC, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_VERT, CTRL_ENABLE);

			arrow_type_set(dlg, IDC_CABLE_DATA_ARROW, ARROW_LEFT_2);
			arrow_type_set(dlg, IDC_CABLE_DATA_JUNC, ARROW_TRUL_2);
			arrow_type_set(dlg, IDC_CABLE_TX_DATA_ARROW, ARROW_HORZ_2);
			arrow_type_set(dlg, IDC_CABLE_RX_DATA_ARROW, ARROW_LEFT_2);
			break;
		}

		// Cable Data is inactive at present.
		ctrl_menu_set(dlg, IDC_CABLE_DATA, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_CABLE_DATA_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_DATA_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_DATA_JUNC, CTRL_DISABLE);

		ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_JUNC, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_TX_DATA_VERT, CTRL_DISABLE);

		ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_ARROW, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_EXT, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_JUNC, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CABLE_RX_DATA_VERT, CTRL_DISABLE);

		arrow_type_set(dlg, IDC_CABLE_DATA_ARROW, ARROW_HORZ_2);
		arrow_type_set(dlg, IDC_CABLE_DATA_JUNC, ARROW_TR_2);
		arrow_type_set(dlg, IDC_CABLE_TX_DATA_ARROW, ARROW_HORZ_2);
		arrow_type_set(dlg, IDC_CABLE_RX_DATA_ARROW, ARROW_HORZ_2);
		break;
	}

	control	= GetDlgItem(dlg, IDC_CABLE_DATA_ARROW);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	cable_data_service
*
*	Purpose:
*
*		Implement the common service entry point for the Cable Data services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT cable_data_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


