// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/open.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement Device Open services.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "open.h"
#include "resource.h"
#include "rx.h"
#include "tx.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			// Force the open/close button to the OPEN function.
			control	= GetDlgItem(dlg, IDC_CLOSE);
			SetWindowLong(control, GWL_ID, IDC_OPEN);
			SetWindowText(control, "Open");

			if (device->count == 0)
			{
				// No devices are present.
				ctrl_menu_set(dlg, IDC_OPEN, CTRL_DISABLE);
				break;
			}

			// Devices are available.
			ctrl_menu_set(dlg, IDC_OPEN, CTRL_ENABLE);
			break;
		}

		// The device is open.
		ctrl_menu_set(dlg, IDC_OPEN, CTRL_DISABLE);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[63];
	HWND	control;
	HCURSOR	cursor;
	U32		status;

	cursor	= SetCursor(device->cursor.wait);

	control	= GetDlgItem(dlg, IDC_SELECT);
	device->index	= (U8) SendMessage(control, CB_GETCURSEL, 0, 0);
	sprintf(buf, "Opening device #%d ...", (int) device->index);
	main_refresh(device, buf);
	status	= hpdi32_open(device->index, &device->handle);

	if ((status == GSC_SUCCESS) && (device->handle))
	{
		sprintf(buf, "Opened device #%d.", (int) device->index);
		tx_oscillator_service(dlg, device, SERVICE_DEVICE_OPEN, 0, 0);
		tx_io_service(dlg, device, SERVICE_DEVICE_OPEN, 0, 0);
		rx_io_service(dlg, device, SERVICE_DEVICE_OPEN, 0, 0);
	}
	else
	{
		sprintf(buf,
				"Device #%d open failure: 0x%lX",
				(int) device->index,
				(long) status);
	}

	main_refresh(device, buf);
	control	= GetDlgItem(dlg, IDC_CLOSE);
	SetFocus(control);
	SetCursor(cursor);
	return(0);
}



/*****************************************************************************
*
*	Function:	open_service
*
*	Purpose:
*
*		Implement the common service entry point for the Device Open services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT open_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


