// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/regs_fw.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement the Firmware Registers dialog.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "regs.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"registers: firmware"



/* typedefs	*****************************************************************/

typedef struct
{
	device_t*	device;
	HWND		dlg;
	void*		last_handle;
} fw_t;



/* variables	*************************************************************/

static struct
{
	int			group;	// Id of group control.
	U32			value;
	int			txt_id;
	int			val_id;
	U32			fsr_bit;
	int			changed;
	reg_def_t*	def;
	reg_t		detail;	// MUST BE LAST, FOR NOW!
} _fw_regs[]	=
{
	{
		/* group	*/	IDC_REGS_FW_ID,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_FRR,
		/* val_id	*/	IDC_REGS_FW_FRR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_frr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_ID,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_FSR,
		/* val_id	*/	IDC_REGS_FW_FSR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_fsr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_CTRL_STATUS,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_BCR,
		/* val_id	*/	IDC_REGS_FW_BCR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_bcr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_CTRL_STATUS,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_BSR,
		/* val_id	*/	IDC_REGS_FW_BSR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_bsr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_TX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_TAR,
		/* val_id	*/	IDC_REGS_FW_TAR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_tar,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_TX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_TFSR,
		/* val_id	*/	IDC_REGS_FW_TFSR_VALUE,
		/* fsr_bit	*/	HPDI32_FSR_TFSR_RFSR,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_tfsr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_TX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_TFWR,
		/* val_id	*/	IDC_REGS_FW_TFWR_VALUE,
		/* fsr_bit	*/	HPDI32_FSR_TFWR_RFWR,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_tfwr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_TX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_TSVLCR,
		/* val_id	*/	IDC_REGS_FW_TSVLCR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_tsvlcr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_TX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_TLVLCR,
		/* val_id	*/	IDC_REGS_FW_TLVLCR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_tlvlcr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_TX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_TLILCR,
		/* val_id	*/	IDC_REGS_FW_TLILCR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_tlilcr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_TX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_TCDR,
		/* val_id	*/	IDC_REGS_FW_TCDR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_tcdr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_RX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_RAR,
		/* val_id	*/	IDC_REGS_FW_RAR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_rar,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_RX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_RFSR,
		/* val_id	*/	IDC_REGS_FW_RFSR_VALUE,
		/* fsr_bit	*/	HPDI32_FSR_TFSR_RFSR,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_rfsr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_RX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_RFWR,
		/* val_id	*/	IDC_REGS_FW_RFWR_VALUE,
		/* fsr_bit	*/	HPDI32_FSR_TFWR_RFWR,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_rfwr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_RX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_RSCR,
		/* val_id	*/	IDC_REGS_FW_RSCR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_rscr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_RX,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_RLCR,
		/* val_id	*/	IDC_REGS_FW_RLCR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_rlcr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_IRQ,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_ICR,
		/* val_id	*/	IDC_REGS_FW_ICR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_icr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_IRQ,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_ISR,
		/* val_id	*/	IDC_REGS_FW_ISR_VALUE,
		/* fsr_bit	*/	0,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_isr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_IRQ,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_IELR,
		/* val_id	*/	IDC_REGS_FW_IELR_VALUE,
		/* fsr_bit	*/	HPDI32_FSR_IELR_IHLR,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_ielr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	},
	{
		/* group	*/	IDC_REGS_FW_IRQ,
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_FW_IHLR,
		/* val_id	*/	IDC_REGS_FW_IHLR_VALUE,
		/* fsr_bit	*/	HPDI32_FSR_IELR_IHLR,
		/* changed	*/	FALSE,
		/* def		*/	&reg_fw_ihlr,
		/* detail	*/	{ 0 /*lint !e785 */ }
	}
};



/*****************************************************************************
*
*	Function:	_get_reg_info
*
*	Purpose:
*
*		Retrieve the info for the given register index and put it in the
*		given buffer.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*		index	The index of the desired register.
*
*		buf		Put the text here.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _get_reg_info(HWND dlg, fw_t* data, int index, char* buf)
{
	HWND	control	= GetDlgItem(dlg, _fw_regs[index].txt_id);
	int		enabled	= IsWindowEnabled(control);
	int		offset	= GSC_REG_OFFSET(_fw_regs[index].def->reg_id);
	int		size	= GSC_REG_SIZE(_fw_regs[index].def->reg_id);

	sprintf(buf,
			"%s: %s: offset 0x%lX, size %d-bits",
			_fw_regs[index].def->name,
			_fw_regs[index].def->desc,
			(long) offset,
			size * 8);

	if ((data->device->handle) && (enabled == FALSE))
		strcat(buf, " (not available in firmware)");
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the content of the window per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	char			buf[16];
	int				color;
	fw_t*			data	= (fw_t*) GET_WINDOW_USER_DATA(dlg);
	unsigned int	flags;
	U32				frr;
	U32				fsr;
	int				i;
	U32				stat;
	U32				status	= GSC_SUCCESS;
	U32				u32;

	set_win_title(dlg, data->device, FALSE, TRUE, "Firmware Registers");
	color	= ((data->last_handle) && (device->handle)) ? TRUE : FALSE;

	if (data->device->handle)
	{
		stat	= hpdi32_reg_read(data->device->handle, HPDI32_FRR, &frr);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= hpdi32_reg_read(data->device->handle, HPDI32_FSR, &fsr);
		status	= (status == GSC_SUCCESS) ? stat : status;

		if ((frr & HPDI32_FRR_FSR) == 0)
			fsr	= 0;
	}
	else
	{
		frr		= 0;
		fsr		= 0;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	if (data->device->handle)
	{
		ctrl_menu_set(dlg, IDC_REFRESH, CTRL_ENABLE);

		if (msg == NULL)
		{
			msg	= "Click on labels for register information."
				  " Double click for details.";
		}

		for (i = 0; i < (int) ARRAY_ELEMENTS(_fw_regs); i++)
		{
			if ((_fw_regs[i].fsr_bit) && (_fw_regs[i].fsr_bit & fsr))
				flags	= CTRL_ENABLE;
			else if (_fw_regs[i].fsr_bit == 0)
				flags	= CTRL_ENABLE;
			else
				flags	= CTRL_DISABLE;

			stat	= hpdi32_reg_read(	data->device->handle,
										_fw_regs[i].def->reg_id,
										&u32);
			status	= (status == GSC_SUCCESS) ? stat : status;

			if ((_fw_regs[i].fsr_bit) && ((_fw_regs[i].fsr_bit & fsr) == 0))
			{
				strcpy(buf, "(NA)");
				_fw_regs[i].changed	= FALSE;
			}
			else if ((color) && (u32 != _fw_regs[i].value))
			{
				sprintf(buf, "%08lX", (long) u32);
				_fw_regs[i].changed	= TRUE;
			}
			else
			{
				sprintf(buf, "%08lX", (long) u32);
				_fw_regs[i].changed	= FALSE;
			}

			ctrl_menu_set(dlg, _fw_regs[i].val_id, flags);
			ctrl_menu_set_text(dlg, _fw_regs[i].val_id, buf);
			_fw_regs[i].value	= u32;
		}
	}
	else
	{
		ctrl_menu_set(dlg, IDC_REFRESH, CTRL_DISABLE);

		if (msg == NULL)
			msg	= "Can't access registers without an open device.";

		for (i = 0; i < (int) ARRAY_ELEMENTS(_fw_regs); i++)
			ctrl_menu_set(dlg, _fw_regs[i].val_id, CTRL_DISABLE | CTRL_NULL);
	}

	ctrl_menu_set_text(dlg, IDC_REGS_FW_STATUS, msg);
	error_status(dlg, IDC_REGS_FW_STATUS, status);
	data->last_handle	= device->handle;
}



/*****************************************************************************
*
*	Function:	_copy_process
*
*	Purpose:
*
*		Perform some processing for the COPY option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		psz			Copy data to this buffer. If NULL then we compute the
*					size of the buffer needed.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		The overall length of the buffer needed,
*
*****************************************************************************/

static size_t _copy_process(HWND dlg, char* psz, fw_t* data)
{
	static const copy_t	top[]	=
	{
		{ -1, COPY_EOL | COPY_UNDERLINE }
	};

	static const copy_t	end[]	=
	{
		{ 0,					COPY_UNDERLINE	},
		{ 0,					COPY_EOL		},

		{ IDC_REFRESH,			COPY_EOL		},
		{ IDCANCEL,				COPY_EOL		},

		{ 0,					COPY_UNDERLINE	},
		{ 0,					COPY_EOL		},

		{ IDC_REGS_FW_STATUS,	COPY_EOL		}
	};

	static const int	list[]	=
	{
		IDC_REGS_FW_ID,
		IDC_REGS_FW_CTRL_STATUS,
		IDC_REGS_FW_TX,
		IDC_REGS_FW_RX,
		IDC_REGS_FW_IRQ
	};

	char			buf[1024];
	copy_t			copy;
	int				i;
	int				j;
	size_t			length	= 0;

	if (psz)
		psz	= copy_list(dlg, psz, top, ARRAY_ELEMENTS(top));
	else
		length	+= copy_list_get_size(dlg, top, ARRAY_ELEMENTS(top));

	for (i = 0; i < (int) ARRAY_ELEMENTS(list); i++)
	{
		copy.id		= list[i];
		copy.flags	= COPY_EOL | COPY_DISABLED |  COPY_WIDTH | 15;

		if (psz)
			psz	= copy_list(dlg, psz, &copy, 1);
		else
			length	+= copy_list_get_size(dlg, &copy, 1);

		for (j = 0; j < (int) ARRAY_ELEMENTS(_fw_regs); j++)
		{
			if (_fw_regs[j].group != list[i])
				continue;

			copy.id		= _fw_regs[j].txt_id;
			copy.flags	= COPY_WIDTH | 8 | COPY_INDENT;

			if (psz)
				psz	= copy_list(dlg, psz, &copy, 1);
			else
				length	+= copy_list_get_size(dlg, &copy, 1);

			copy.id		= _fw_regs[j].val_id;
			copy.flags	= COPY_WIDTH | 10;

			if (psz)
				psz	= copy_list(dlg, psz, &copy, 1);
			else
				length	+= copy_list_get_size(dlg, &copy, 1);

			_get_reg_info(dlg, data, j, buf);

			if (psz)
			{
				strcpy(psz, buf);
				strcat(psz, "\r\n");
				psz	+= strlen(psz);
			}
			else
			{
				length	+= strlen(buf) + 4;
			}
		}
	}

	if (psz)
		copy_list(dlg, psz, end, ARRAY_ELEMENTS(end));
	else
		length	+= copy_list_get_size(dlg, end, ARRAY_ELEMENTS(end));

	return(length);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg, fw_t* data)
{
	GLOBALHANDLE	gh;
	size_t			length;
	char*			psz;

	length	= _copy_process(dlg, NULL, data);
	psz		= copy_alloc(dlg, length, &gh);

	if (psz)
	{
		_copy_process(dlg, psz, data);
		copy_perform(dlg, gh);
	}
}



/*****************************************************************************
*
*	Function:	_idc_regs_fw_xxx
*
*	Purpose:
*
*		Respond to activity on the given register name text control.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		wParam		The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_regs_fw_xxx(HWND dlg, fw_t* data, WPARAM wParam)
{
	char	buf[256];
	int		code	= HIWORD(wParam);
	int		i;
	int		id		= LOWORD(wParam);

	for (i = ARRAY_ELEMENTS(_fw_regs); i--;)
	{
		if (id == _fw_regs[i].txt_id)
			break;
	}

	if (i < 0)
	{
	}
	else if (code == STN_CLICKED)
	{
		_get_reg_info(dlg, data, i, buf);
		ctrl_menu_set_text(dlg,  IDC_REGS_FW_STATUS, buf);
	}
	else if ((data->device->handle) && (code == STN_DBLCLK))
	{
		_get_reg_info(dlg, data, i, buf);
		ctrl_menu_set_text(dlg,  IDC_REGS_FW_STATUS, buf);
		dlg	= GetParent(dlg);
		_fw_regs[i].detail.def	= _fw_regs[i].def;
		reg_service(dlg,
					data->device,
					SERVICE_WM_COMMAND,
					0,
					(ULONG) &_fw_regs[i].detail);
	}
}



/*****************************************************************************
*
*	Function:	_idc_regs_fw_xxx_value
*
*	Purpose:
*
*		Respond to activity on the given register value text control.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		wParam		The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_regs_fw_xxx_value(HWND dlg, fw_t* data, WPARAM wParam)
{
	char	buf[256];
	int		code	= HIWORD(wParam);
	int		i;
	int		id		= LOWORD(wParam);

	for (i = ARRAY_ELEMENTS(_fw_regs); i--;)
	{
		if (id == _fw_regs[i].val_id)
			break;
	}

	if (i < 0)
	{
	}
	else if (code == STN_CLICKED)
	{
		_get_reg_info(dlg, data, i, buf);
		ctrl_menu_set_text(dlg,  IDC_REGS_FW_STATUS, buf);
	}
	else if ((data->device->handle) && (code == STN_DBLCLK))
	{
		_get_reg_info(dlg, data, i, buf);
		ctrl_menu_set_text(dlg,  IDC_REGS_FW_STATUS, buf);
		dlg	= GetParent(dlg);
		_fw_regs[i].detail.def	= _fw_regs[i].def;
		reg_service(dlg,
					data->device,
					SERVICE_WM_COMMAND,
					0,
					(ULONG) &_fw_regs[i].detail);
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	fw_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (fw_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	fw_t*	data	= (fw_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_COPY:

			_idc_copy(dlg, data);
			break;

		case IDC_REFRESH:

			_refresh_window(dlg, data->device, NULL);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDC_REGS_FW_FRR:
		case IDC_REGS_FW_FSR:
		case IDC_REGS_FW_BCR:
		case IDC_REGS_FW_BSR:
		case IDC_REGS_FW_TAR:
		case IDC_REGS_FW_TFSR:
		case IDC_REGS_FW_TFWR:
		case IDC_REGS_FW_TSVLCR:
		case IDC_REGS_FW_TLVLCR:
		case IDC_REGS_FW_TLILCR:
		case IDC_REGS_FW_TCDR:
		case IDC_REGS_FW_RAR:
		case IDC_REGS_FW_RFSR:
		case IDC_REGS_FW_RFWR:
		case IDC_REGS_FW_RSCR:
		case IDC_REGS_FW_RLCR:
		case IDC_REGS_FW_ICR:
		case IDC_REGS_FW_ISR:
		case IDC_REGS_FW_IELR:
		case IDC_REGS_FW_IHLR:

			_idc_regs_fw_xxx(dlg, data, wParam);
			break;

		case IDC_REGS_FW_FRR_VALUE:
		case IDC_REGS_FW_FSR_VALUE:
		case IDC_REGS_FW_BCR_VALUE:
		case IDC_REGS_FW_BSR_VALUE:
		case IDC_REGS_FW_TAR_VALUE:
		case IDC_REGS_FW_TFSR_VALUE:
		case IDC_REGS_FW_TFWR_VALUE:
		case IDC_REGS_FW_TSVLCR_VALUE:
		case IDC_REGS_FW_TLVLCR_VALUE:
		case IDC_REGS_FW_TLILCR_VALUE:
		case IDC_REGS_FW_TCDR_VALUE:
		case IDC_REGS_FW_RAR_VALUE:
		case IDC_REGS_FW_RFSR_VALUE:
		case IDC_REGS_FW_RFWR_VALUE:
		case IDC_REGS_FW_RSCR_VALUE:
		case IDC_REGS_FW_RLCR_VALUE:
		case IDC_REGS_FW_ICR_VALUE:
		case IDC_REGS_FW_ISR_VALUE:
		case IDC_REGS_FW_IELR_VALUE:
		case IDC_REGS_FW_IHLR_VALUE:

			_idc_regs_fw_xxx_value(dlg, data, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_ctlcolorstatic
*
*	Purpose:
*
*		Process a WM_CTLCOLORSTATIC message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The WORD sized parameter for the message.
*
*		lParam		The LONG sized parameter for the message.
*
*	Returned:
*
*		The brush handle to use for the background.
*
*****************************************************************************/

static LRESULT _wm_ctlcolorstatic(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	fw_t*		data;
	HDC			dc		= (HDC) wParam;
	HWND		cntrl;
	HWND		control	= (HWND) lParam;
	int			i;
	HGDIOBJ		obj		= NULL;

	data	= (fw_t*) GET_WINDOW_USER_DATA(dlg);

	for (i = 0; i < (int) ARRAY_ELEMENTS(_fw_regs); i++)
	{
		cntrl	= GetDlgItem(dlg, _fw_regs[i].val_id);

		if (cntrl == control)
		{
			obj		= data->device->paint.btnface.brush;

			if (_fw_regs[i].changed)
				SetTextColor(dc, data->device->paint.red.color);
			else
				SetTextColor(dc, data->device->paint.window_text.color);

			SetBkColor(dc, data->device->paint.btnface.color);
			break;
		}
	}

	return((LRESULT) obj);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	fw_t*	data	= (fw_t*) lParam;
	int		i;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	for (i = 0; i < (int) ARRAY_ELEMENTS(_fw_regs); i++)
		ctrl_menu_set_text(dlg, _fw_regs[i].txt_id, _fw_regs[i].def->name);

	win_pos_restore(dlg, SECTION_NAME, FALSE);

	_refresh_window(dlg, data->device, NULL);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_CTLCOLORSTATIC:

			ret	= _wm_ctlcolorstatic(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_REGS_FW, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_REGS_FW, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	fw_t	data;

	if (data.dlg)		/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.device			= device;
		data.last_handle	= NULL;
		data.dlg			= CreateDialogParam(device->instance,
												(LPCTSTR) IDD_REGS_FW,
												dlg,
												(DLGPROC) _dlg_proc,
												(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	regs_fw_service
*
*	Purpose:
*
*		Implement the common service entry point for the Firmware Register
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT regs_fw_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


