// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/regs_fw_fsr.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

/*****************************************************************************
*
*	Provide data describing the Feature Set Register.
*
*****************************************************************************/

#include "regs.h"



/* variables	*************************************************************/

static const reg_value_t	_dmdma1_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_DMDMA_CH1
	}
};

static const reg_value_t	_fsr_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_TFSR_RFSR
	}
};

static const reg_value_t	_fwr_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_TFWR_RFWR
	}
};

static const reg_value_t	_g05_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_GPIO_0_5
	}
};

static const reg_value_t	_g6tas_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_GPIO6_TXAS
	}
};

static const reg_value_t	_icr_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_IELR_IHLR
	}
};

static const reg_value_t	_our_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_OVR_UNDR_RUN
	}
};

static const reg_value_t	_scd_values[]	=
{
	{
	/* name		*/	"No",
	/* value	*/	0
	},
	{
	/* name		*/	"Yes",
	/* value	*/	HPDI32_FSR_SCD
	}
};

static reg_field_t	_fields[]	=
{
	{
	/* name		*/	"RES",
	/* desc		*/	"Reserved",
	/* mask		*/	0xFFFFFF00,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"SCD",
	/* desc		*/	"Single Cycle Disable",
	/* mask		*/	HPDI32_FSR_SCD,
	/* value	*/	_scd_values,
	/* values	*/	ARRAY_ELEMENTS(_scd_values)
	},
	{
	/* name		*/	"G6TAS",
	/* desc		*/	"GPIO 6/Tx Auto Start",
	/* mask		*/	HPDI32_FSR_GPIO6_TXAS,
	/* value	*/	_g6tas_values,
	/* values	*/	ARRAY_ELEMENTS(_g6tas_values)
	},
	{
	/* name		*/	"OUR",
	/* desc		*/	"Tx/Rx Over/Under Run",
	/* mask		*/	HPDI32_FSR_OVR_UNDR_RUN,
	/* value	*/	_our_values,
	/* values	*/	ARRAY_ELEMENTS(_our_values)
	},
	{
	/* name		*/	"DMDMA1",
	/* desc		*/	"Tx DMDMA on DMA channel 1",
	/* mask		*/	HPDI32_FSR_DMDMA_CH1,
	/* value	*/	_dmdma1_values,
	/* values	*/	ARRAY_ELEMENTS(_dmdma1_values)
	},
	{
	/* name		*/	"G05",
	/* desc		*/	"GPIO 0-5",
	/* mask		*/	HPDI32_FSR_GPIO_0_5,
	/* value	*/	_g05_values,
	/* values	*/	ARRAY_ELEMENTS(_g05_values)
	},
	{
	/* name		*/	"ICR",
	/* desc		*/	"Interrupt Configuration Registers: IELR & IHLR",
	/* mask		*/	HPDI32_FSR_IELR_IHLR,
	/* value	*/	_icr_values,
	/* values	*/	ARRAY_ELEMENTS(_icr_values)
	},
	{
	/* name		*/	"FWR",
	/* desc		*/	"Tx/Rx FIFO Words Registers",
	/* mask		*/	HPDI32_FSR_TFWR_RFWR,
	/* value	*/	_fwr_values,
	/* values	*/	ARRAY_ELEMENTS(_fwr_values)
	},
	{
	/* name		*/	"FSR",
	/* desc		*/	"Tx/Rx FIFO Size Registers",
	/* mask		*/	HPDI32_FSR_TFSR_RFSR,
	/* value	*/	_fsr_values,
	/* values	*/	ARRAY_ELEMENTS(_fsr_values)
	}
};

reg_def_t	reg_fw_fsr	=
{
	/* name			*/	"FSR",
	/* desc			*/	"Feature Set Register",
	/* reg_id		*/	HPDI32_FSR,
	/* read_only	*/	0xFFFFFFFF,
	/* field		*/	_fields,
	/* fields		*/	ARRAY_ELEMENTS(_fields),
	/* init			*/	NULL,
	/* refresh		*/	NULL
};


