// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/regs_pci.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement the PCI Registers dialog.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "regs.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"registers: pci"



/* typedefs	*****************************************************************/

typedef struct
{
	device_t*	device;
	HWND		dlg;
	void*		last_handle;
} pci_t;



/* variables	*************************************************************/

static struct
{
	int			group;	// The group box the register is in.
	U32			bus_size;
	U32			reg_id;
	const char*	name;
	U32			value;
	int			txt_id;
	int			val_id;
	int			changed;
} _pci_regs[]	=
{
	{
		/* group	*/	IDC_REGS_PCI_ID,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_VIDR,
		/* name		*/	"Vendor ID Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_VIDR,
		/* val_id	*/	IDC_REGS_PCI_VIDR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_ID,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_DIDR,
		/* name		*/	"Device ID Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_DIDR,
		/* val_id	*/	IDC_REGS_PCI_DIDR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_ID,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_SVID,
		/* name		*/	"Subsystem Vendor ID Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_SVID,
		/* val_id	*/	IDC_REGS_PCI_SVID_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_ID,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_SID,
		/* name		*/	"Subsystem ID Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_SID,
		/* val_id	*/	IDC_REGS_PCI_SID_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_ID,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_REV,
		/* name		*/	"Revision ID Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_REV,
		/* val_id	*/	IDC_REGS_PCI_REV_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_ID,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_CCR,
		/* name		*/	"Class Code Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_CCR,
		/* val_id	*/	IDC_REGS_PCI_CCR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_ID,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_HTR,
		/* name		*/	"Header Type Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_HTR,
		/* val_id	*/	IDC_REGS_PCI_HTR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_BAR,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_BAR0,
		/* name		*/	"Base Address Register 0",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_BAR0,
		/* val_id	*/	IDC_REGS_PCI_BAR0_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_BAR,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_BAR1,
		/* name		*/	"Base Address Register 1",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_BAR1,
		/* val_id	*/	IDC_REGS_PCI_BAR1_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_BAR,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_BAR2,
		/* name		*/	"Base Address Register 2",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_BAR2,
		/* val_id	*/	IDC_REGS_PCI_BAR2_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_BAR,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_BAR3,
		/* name		*/	"Base Address Register 3",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_BAR3,
		/* val_id	*/	IDC_REGS_PCI_BAR3_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_BAR,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_BAR4,
		/* name		*/	"Base Address Register 4",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_BAR4,
		/* val_id	*/	IDC_REGS_PCI_BAR4_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_BAR,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_BAR5,
		/* name		*/	"Base Address Register 5",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_BAR5,
		/* val_id	*/	IDC_REGS_PCI_BAR5_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_BAR,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_ERBAR,
		/* name		*/	"Expansion ROM Base Address Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_ERBAR,
		/* val_id	*/	IDC_REGS_PCI_ERBAR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CNTROL_STATUS,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_BISTR,
		/* name		*/	"Built-In Self Test Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_BISTR,
		/* val_id	*/	IDC_REGS_PCI_BISTR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CNTROL_STATUS,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_CR,
		/* name		*/	"Command Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_CR,
		/* val_id	*/	IDC_REGS_PCI_CR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CNTROL_STATUS,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_SR,
		/* name		*/	"Status Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_SR,
		/* val_id	*/	IDC_REGS_PCI_SR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CNTROL_STATUS,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_CIS,
		/* name		*/	"Cardbus CIS Pointer Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_CIS,
		/* val_id	*/	IDC_REGS_PCI_CIS_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CONFIG,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_IPR,
		/* name		*/	"Interrupt Pin Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_IPR,
		/* val_id	*/	IDC_REGS_PCI_IPR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CONFIG,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_ILR,
		/* name		*/	"Interrupt Line Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_ILR,
		/* val_id	*/	IDC_REGS_PCI_ILR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CONFIG,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_CLSR,
		/* name		*/	"Cache Line Size Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_CLSR,
		/* val_id	*/	IDC_REGS_PCI_CLSR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CONFIG,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_LTR,
		/* name		*/	"PCI Latency Timer Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_LTR,
		/* val_id	*/	IDC_REGS_PCI_LTR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CONFIG,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_MGR,
		/* name		*/	"Min_Gnt Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_MGR,
		/* val_id	*/	IDC_REGS_PCI_MGR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_CONFIG,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9080_MLR,
		/* name		*/	"Max_Lat Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_MLR,
		/* val_id	*/	IDC_REGS_PCI_MLR_VALUE,
		/* changed	*/	FALSE
	},
	{	// This is on both, but is unused on the PCI9080.
		/* group	*/	IDC_REGS_PCI_CONFIG,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_32,
		/* reg_id	*/	GSC_PCI_9656_CAP_PTR,
		/* name		*/	"New Capability Pointer Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_CAP_PTR,
		/* val_id	*/	IDC_REGS_PCI_CAP_PTR_VALUE,
		/* changed	*/	FALSE
	},

	// These are supported only on the 64-bit boards.
	{
		/* group	*/	IDC_REGS_PCI_POWER_MAN,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_PMCAPID,
		/* name		*/	"Power Management Capability ID Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_PMCAPID,
		/* val_id	*/	IDC_REGS_PCI_PMCAPID_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_POWER_MAN,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_PMNEXT,
		/* name		*/	"Power Management Next Capability Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_PMNEXT,
		/* val_id	*/	IDC_REGS_PCI_PMNEXT_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_POWER_MAN,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_PMC,
		/* name		*/	"Power Management Capabilities Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_PMC,
		/* val_id	*/	IDC_REGS_PCI_PMC_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_POWER_MAN,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_PMCSR,
		/* name		*/	"Power Management Control/Status Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_PMCSR,
		/* val_id	*/	IDC_REGS_PCI_PMCSR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_POWER_MAN,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_PMCSR_BSE,
		/* name		*/	"PMCSR Bridge Support Extensions Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_PMCSR_BSE,
		/* val_id	*/	IDC_REGS_PCI_PMCSR_BSE_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_POWER_MAN,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_PMDATA,
		/* name		*/	"Power Management Data Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_PMDATA,
		/* val_id	*/	IDC_REGS_PCI_PMDATA_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_VPD,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_VPDIDR,
		/* name		*/	"Vital Product Data Identification Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_VPDIDR,
		/* val_id	*/	IDC_REGS_PCI_VPDIDR_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_VPD,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_VPD_NEXT,
		/* name		*/	"Vital Product Data Next Capability Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_VPD_NEXT,
		/* val_id	*/	IDC_REGS_PCI_VPD_NEXT_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_VPD,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_VPDAD,
		/* name		*/	"Vital Product Data Address Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_VPDAD,
		/* val_id	*/	IDC_REGS_PCI_VPDAD_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_VPD,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_VPDATA,
		/* name		*/	"Vital Product Data Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_VPDATA,
		/* val_id	*/	IDC_REGS_PCI_VPDATA_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_HOT_SWAP,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_HS_CNTL,
		/* name		*/	"Hot Swap Control Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_HS_CNTL,
		/* val_id	*/	IDC_REGS_PCI_HS_CNTL_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_HOT_SWAP,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_HS_NEXT,
		/* name		*/	"Hot Swap Next Capability Pointer Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_HS_NEXT,
		/* val_id	*/	IDC_REGS_PCI_HS_NEXT_VALUE,
		/* changed	*/	FALSE
	},
	{
		/* group	*/	IDC_REGS_PCI_HOT_SWAP,
		/* bus_size	*/	HPDI32_MISC_PCI_BUS_WIDTH_64,
		/* reg_id	*/	GSC_PCI_9656_HS_CSR,
		/* name		*/	"Hot Swap Control/Status Register",
		/* value	*/	0,
		/* txt_id	*/	IDC_REGS_PCI_HS_CSR,
		/* val_id	*/	IDC_REGS_PCI_HS_CSR_VALUE,
		/* changed	*/	FALSE
	}
};



/*****************************************************************************
*
*	Function:	_get_reg_info
*
*	Purpose:
*
*		Retrieve the info for the given register index and put it in the
*		given buffer.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		index		The index of the desired register.
*
*		buf			Put the text here.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _get_reg_info(HWND dlg, pci_t* data, int index, char* buf)
{
	HWND	control	= GetDlgItem(dlg, _pci_regs[index].txt_id);
	int		enabled	= IsWindowEnabled(control);
	int		offset	= GSC_REG_OFFSET(_pci_regs[index].reg_id);
	int		size	= GSC_REG_SIZE(_pci_regs[index].reg_id);

	sprintf(buf,
			"%s: offset 0x%lX, size %d-bits",
			_pci_regs[index].name,
			(long) offset,
			size * 8);

	if ((data->device->handle) && (enabled == FALSE))
		strcat(buf, " (not available on 32-bit boards)");
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the content of the window per the state of the system.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	char	buf[16];
	U32		bw		= 0;
	int		color;
	pci_t*	data	= (pci_t*) GET_WINDOW_USER_DATA(dlg);
	int		i;
	int		size;
	U32		stat;
	U32		status	= GSC_SUCCESS;
	U32		u32;

	color	= ((data->last_handle) && (device->handle)) ? TRUE : FALSE;
	set_win_title(dlg, data->device, FALSE, TRUE, "PCI Registers");

	if (data->device->handle)
	{
		stat	= HPDI32_MISC_PCI_BUS_WIDTH__GET(data->device->handle, &bw);
		status	= (status == GSC_SUCCESS) ? stat : status;
	}
	else
	{
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	if (data->device->handle)
	{
		ctrl_menu_set(dlg, IDC_REFRESH, CTRL_ENABLE);

		if (msg == NULL)
			msg	= "Click on labels for register information.";

		for (i = 0; i < (int) ARRAY_ELEMENTS(_pci_regs); i++)
		{
			if (_pci_regs[i].bus_size > bw)
			{
				_pci_regs[i].changed	= FALSE;
				ctrl_menu_set_text(dlg, _pci_regs[i].val_id, "N/A");
				continue;
			}

			ctrl_menu_set(dlg, _pci_regs[i].val_id, CTRL_ENABLE);

			stat	= hpdi32_reg_read(	data->device->handle,
										_pci_regs[i].reg_id,
										&u32);
			status	= (status == GSC_SUCCESS) ? stat : status;
			size	= GSC_REG_SIZE(_pci_regs[i].reg_id);

			if (size == 4)
				sprintf(buf, "%08lX", (long) u32);
			else if (size == 2)
				sprintf(buf, "%04lX", (long) u32);
			else
				sprintf(buf, "%02lX", (long) u32);

			if ((color) && (u32 != _pci_regs[i].value))
				_pci_regs[i].changed	= TRUE;
			else
				_pci_regs[i].changed	= FALSE;

			ctrl_menu_set_text(dlg, _pci_regs[i].val_id, buf);
			_pci_regs[i].value	= u32;
		}
	}
	else
	{
		ctrl_menu_set(dlg, IDC_REFRESH, CTRL_DISABLE);

		if (msg == NULL)
			msg	= "Can't access registers without an open device.";

		for (i = 0; i < (int) ARRAY_ELEMENTS(_pci_regs); i++)
			ctrl_menu_set(dlg, _pci_regs[i].val_id, CTRL_NULL);
	}

	ctrl_menu_set_text(dlg, IDC_REGS_PCI_STATUS, msg);
	error_status(dlg, IDC_REGS_PCI_STATUS, status);
	data->last_handle	= device->handle;
}



/*****************************************************************************
*
*	Function:	_copy_process
*
*	Purpose:
*
*		Perform some processing for the COPY option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		psz			Copy data to this buffer. If NULL then we compute the
*					size of the buffer needed.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		The overall length of the buffer needed,
*
*****************************************************************************/

static size_t _copy_process(HWND dlg, char* psz, pci_t* data)
{
	static const copy_t	top[]	=
	{
		{ -1, COPY_EOL | COPY_UNDERLINE }
	};

	static const copy_t	end[]	=
	{
		{ 0,					COPY_UNDERLINE	},
		{ 0,					COPY_EOL		},

		{ IDC_REFRESH,			COPY_EOL		},
		{ IDCANCEL,				COPY_EOL		},

		{ 0,					COPY_UNDERLINE	},
		{ 0,					COPY_EOL		},

		{ IDC_REGS_PCI_STATUS,	COPY_EOL		}
	};

	static const int	list[]	=
	{
		IDC_REGS_PCI_ID,
		IDC_REGS_PCI_BAR,
		IDC_REGS_PCI_CNTROL_STATUS,
		IDC_REGS_PCI_CONFIG,
		IDC_REGS_PCI_POWER_MAN,
		IDC_REGS_PCI_VPD,
		IDC_REGS_PCI_HOT_SWAP
	};

	char			buf[1024];
	copy_t			copy;
	int				i;
	int				j;
	size_t			length	= 0;

	if (psz)
		psz	= copy_list(dlg, psz, top, ARRAY_ELEMENTS(top));
	else
		length	+= copy_list_get_size(dlg, top, ARRAY_ELEMENTS(top));

	for (i = 0; i < (int) ARRAY_ELEMENTS(list); i++)
	{
		copy.id		= list[i];
		copy.flags	= COPY_EOL | COPY_DISABLED |  COPY_WIDTH | 15;

		if (psz)
			psz	= copy_list(dlg, psz, &copy, 1);
		else
			length	+= copy_list_get_size(dlg, &copy, 1);

		for (j = 0; j < (int) ARRAY_ELEMENTS(_pci_regs); j++)
		{
			if (_pci_regs[j].group != list[i])
				continue;

			copy.id		= _pci_regs[j].txt_id;
			copy.flags	= COPY_WIDTH | 11 | COPY_INDENT;

			if (psz)
				psz	= copy_list(dlg, psz, &copy, 1);
			else
				length	+= copy_list_get_size(dlg, &copy, 1);

			copy.id		= _pci_regs[j].val_id;
			copy.flags	= COPY_WIDTH | 10;

			if (psz)
				psz	= copy_list(dlg, psz, &copy, 1);
			else
				length	+= copy_list_get_size(dlg, &copy, 1);

			_get_reg_info(dlg, data, j, buf);

			if (psz)
			{
				strcpy(psz, buf);
				strcat(psz, "\r\n");
				psz	+= strlen(psz);
			}
			else
			{
				length	+= strlen(buf) + 4;
			}
		}
	}

	if (psz)
		copy_list(dlg, psz, end, ARRAY_ELEMENTS(end));
	else
		length	+= copy_list_get_size(dlg, end, ARRAY_ELEMENTS(end));

	return(length);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg, pci_t* data)
{
	GLOBALHANDLE	gh;
	size_t			length;
	char*			psz;

	length	= _copy_process(dlg, NULL, data);
	psz		= copy_alloc(dlg, length, &gh);

	if (psz)
	{
		_copy_process(dlg, psz, data);
		copy_perform(dlg, gh);
	}
}



/*****************************************************************************
*
*	Function:	_idc_regs_pci_xxx
*
*	Purpose:
*
*		Respond to activity on the given register name text control.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		wParam		The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_regs_pci_xxx(HWND dlg, pci_t* data, WPARAM wParam)
{
	char	buf[256];
	int		code	= HIWORD(wParam);
	int		i;
	int		id		= LOWORD(wParam);

	for (i = ARRAY_ELEMENTS(_pci_regs); i--;)
	{
		if (id == _pci_regs[i].txt_id)
			break;
	}

	if (i < 0)
	{
	}
	else if (code == STN_CLICKED)
	{
		_get_reg_info(dlg, data, i, buf);
		ctrl_menu_set_text(dlg,  IDC_REGS_PCI_STATUS, buf);
	}
}



/*****************************************************************************
*
*	Function:	_idc_regs_pci_xxx_value
*
*	Purpose:
*
*		Respond to activity on the given register value text control.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*		wParam		The message's WORD sized parameter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_regs_pci_xxx_value(HWND dlg, pci_t* data, WPARAM wParam)
{
	char	buf[256];
	int		code	= HIWORD(wParam);
	int		i;
	int		id		= LOWORD(wParam);

	for (i = ARRAY_ELEMENTS(_pci_regs); i--;)
	{
		if (id == _pci_regs[i].val_id)
			break;
	}

	if (i < 0)
	{
	}
	else if (code == STN_CLICKED)
	{
		_get_reg_info(dlg, data, i, buf);
		ctrl_menu_set_text(dlg,  IDC_REGS_PCI_STATUS, buf);
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	pci_t*	data	= (pci_t*) GET_WINDOW_USER_DATA(dlg);

	main_refresh_add(dlg, NULL);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	pci_t*	data	= (pci_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_COPY:

			_idc_copy(dlg, data);
			break;

		case IDC_REFRESH:

			_refresh_window(dlg, data->device, NULL);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDC_REGS_PCI_VIDR:
		case IDC_REGS_PCI_DIDR:
		case IDC_REGS_PCI_SVID:
		case IDC_REGS_PCI_SID:
		case IDC_REGS_PCI_REV:
		case IDC_REGS_PCI_CCR:
		case IDC_REGS_PCI_HTR:
		case IDC_REGS_PCI_BAR0:
		case IDC_REGS_PCI_BAR1:
		case IDC_REGS_PCI_BAR2:
		case IDC_REGS_PCI_BAR3:
		case IDC_REGS_PCI_BAR4:
		case IDC_REGS_PCI_BAR5:
		case IDC_REGS_PCI_ERBAR:
		case IDC_REGS_PCI_BISTR:
		case IDC_REGS_PCI_CR:
		case IDC_REGS_PCI_SR:
		case IDC_REGS_PCI_CIS:
		case IDC_REGS_PCI_IPR:
		case IDC_REGS_PCI_ILR:
		case IDC_REGS_PCI_CLSR:
		case IDC_REGS_PCI_LTR:
		case IDC_REGS_PCI_MGR:
		case IDC_REGS_PCI_MLR:
		case IDC_REGS_PCI_CAP_PTR:
		case IDC_REGS_PCI_PMCAPID:	// These are 64-bit specific.
		case IDC_REGS_PCI_PMNEXT:
		case IDC_REGS_PCI_PMC:
		case IDC_REGS_PCI_PMCSR:
		case IDC_REGS_PCI_PMCSR_BSE:
		case IDC_REGS_PCI_PMDATA:
		case IDC_REGS_PCI_VPDIDR:
		case IDC_REGS_PCI_VPD_NEXT:
		case IDC_REGS_PCI_VPDAD:
		case IDC_REGS_PCI_VPDATA:
		case IDC_REGS_PCI_HS_CNTL:
		case IDC_REGS_PCI_HS_NEXT:
		case IDC_REGS_PCI_HS_CSR:

			_idc_regs_pci_xxx(dlg, data, wParam);
			break;

		case IDC_REGS_PCI_VIDR_VALUE:
		case IDC_REGS_PCI_DIDR_VALUE:
		case IDC_REGS_PCI_SVID_VALUE:
		case IDC_REGS_PCI_SID_VALUE:
		case IDC_REGS_PCI_REV_VALUE:
		case IDC_REGS_PCI_CCR_VALUE:
		case IDC_REGS_PCI_HTR_VALUE:
		case IDC_REGS_PCI_BAR0_VALUE:
		case IDC_REGS_PCI_BAR1_VALUE:
		case IDC_REGS_PCI_BAR2_VALUE:
		case IDC_REGS_PCI_BAR3_VALUE:
		case IDC_REGS_PCI_BAR4_VALUE:
		case IDC_REGS_PCI_BAR5_VALUE:
		case IDC_REGS_PCI_ERBAR_VALUE:
		case IDC_REGS_PCI_BISTR_VALUE:
		case IDC_REGS_PCI_CR_VALUE:
		case IDC_REGS_PCI_SR_VALUE:
		case IDC_REGS_PCI_CIS_VALUE:
		case IDC_REGS_PCI_IPR_VALUE:
		case IDC_REGS_PCI_ILR_VALUE:
		case IDC_REGS_PCI_CLSR_VALUE:
		case IDC_REGS_PCI_LTR_VALUE:
		case IDC_REGS_PCI_MGR_VALUE:
		case IDC_REGS_PCI_MLR_VALUE:
		case IDC_REGS_PCI_CAP_PTR_VALUE:
		case IDC_REGS_PCI_PMCAPID_VALUE:	// These are 64-bit specific.
		case IDC_REGS_PCI_PMNEXT_VALUE:
		case IDC_REGS_PCI_PMC_VALUE:
		case IDC_REGS_PCI_PMCSR_VALUE:
		case IDC_REGS_PCI_PMCSR_BSE_VALUE:
		case IDC_REGS_PCI_PMDATA_VALUE:
		case IDC_REGS_PCI_VPDIDR_VALUE:
		case IDC_REGS_PCI_VPD_NEXT_VALUE:
		case IDC_REGS_PCI_VPDAD_VALUE:
		case IDC_REGS_PCI_VPDATA_VALUE:
		case IDC_REGS_PCI_HS_CNTL_VALUE:
		case IDC_REGS_PCI_HS_NEXT_VALUE:
		case IDC_REGS_PCI_HS_CSR_VALUE:

			_idc_regs_pci_xxx_value(dlg, data, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_ctlcolorstatic
*
*	Purpose:
*
*		Process a WM_CTLCOLORSTATIC message for the dialog.
*
*	Arguments:
*
*		dlg			The window handle.
*
*		wParam		The WORD sized parameter for the message.
*
*		lParam		The LONG sized parameter for the message.
*
*	Returned:
*
*		The brush handle to use for the background.
*
*****************************************************************************/

static LRESULT _wm_ctlcolorstatic(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	pci_t*	data	= (pci_t*) GET_WINDOW_USER_DATA(dlg);
	HDC		dc		= (HDC) wParam;
	HWND	cntrl;
	HWND	control	= (HWND) lParam;
	int		i;
	HGDIOBJ	obj		= NULL;

	for (i = 0; i < (int) ARRAY_ELEMENTS(_pci_regs); i++)
	{
		cntrl	= GetDlgItem(dlg, _pci_regs[i].val_id);

		if (cntrl == control)
		{
			obj	= data->device->paint.btnface.brush;

			if (_pci_regs[i].changed)
				SetTextColor(dc, data->device->paint.red.color);
			else
				SetTextColor(dc, data->device->paint.window_text.color);

			SetBkColor(dc, data->device->paint.btnface.color);
			break;
		}
	}

	return((LRESULT) obj);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The messages LONG sized parameter.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	pci_t*	data	= (pci_t*) lParam;
	int		i;

	SET_WINDOW_USER_DATA(dlg, lParam);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	for (i = 0; i < (int) ARRAY_ELEMENTS(_pci_regs); i++)
		_pci_regs[i].changed	= FALSE;

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	_refresh_window(dlg, data->device, NULL);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_CTLCOLORSTATIC:

			ret	= _wm_ctlcolorstatic(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_REGS_PCI, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_REGS_PCI, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	pci_t	data;

	if (data.dlg)		/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.device			= device;
		data.last_handle	= NULL;
		data.dlg			= CreateDialogParam(device->instance,
												(LPCTSTR) IDD_REGS_PCI,
												dlg,
												(DLGPROC) _dlg_proc,
												(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	regs_pci_service
*
*	Purpose:
*
*		Implement the common service entry point for the PCI Register services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT regs_pci_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


