// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Receiver Parameters.
*
*****************************************************************************/

#include <windows.h>

#include "rx.h"
#include "align.h"
#include "arrow.h"
#include "io.h"
#include "resource.h"
#include "util_ctrl.h"



/* variables	*************************************************************/

static	const parm_entry_t*	_rx_parm_list[]	=
{
	&rx_parm_enable
};

const parm_list_t	rx_parm_list	=
{
	/* list		*/	_rx_parm_list,
	/* size		*/	ARRAY_ELEMENTS(_rx_parm_list),
	/* suffix	*/	NULL,
	/* which	*/	0
};

const parm_list_t	rx_io_parm_list	=
{
	/* list		*/	io_parm_list,
	/* size		*/	ARRAY_ELEMENTS(io_parm_list),
	/* suffix	*/	"_RX",
	/* which	*/	HPDI32_WHICH_RX
};

const parm_list_t	rx_io_parm_api_buffer_list	=
{
	/* list		*/	io_parm_api_buffer_list,
	/* size		*/	ARRAY_ELEMENTS(io_parm_api_buffer_list),
	/* suffix	*/	"_RX",
	/* which	*/	HPDI32_WHICH_RX
};



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_RX,
		/* ref		*/	IDC_RX_IO,
		/* flags	*/	ALIGN_L_L | ALIGN_T_B,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX,
		/* ref		*/	IDC_RX_FIFO,
		/* flags	*/	ALIGN_R_L,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX,
		/* ref		*/	IDC_RX_SV,
		/* flags	*/	ALIGN_B_T,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX,
		/* ref		*/	IDC_RX,
		/* flags	*/	ALIGN_T_C,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	HWND	control;

	control	= GetDlgItem(dlg, IDC_RX);
	SendMessage(control, WM_SETFONT, (WPARAM) device->font.defalt_bl, 0);
	align_control_list(dlg, list);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_RX, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_RX, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_ctlcolorstatic
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_CTLCOLORSTATIC
*		service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		Use the default brush.
*		else	Use this brush handle for the background.
*
*****************************************************************************/

static LRESULT _service_wm_ctlcolorstatic(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HDC		dc	= (HDC) wParam;
	LRESULT	ret	= (LRESULT) device->paint.btnhighlt.brush;

	SetBkColor(dc, device->paint.btnhighlt.color);
	return(ret);
}



/*****************************************************************************
*
*	Function:	rx_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service						function					*/
		{ SERVICE_INIT,					_service_init				},
		{ SERVICE_REFRESH,				_service_refresh			},
		{ SERVICE_WM_CTLCOLORSTATIC,	_service_wm_ctlcolorstatic	},
		{ SERVICE_END_LIST,				NULL						}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


