// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx_fifo.c $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Implement support for the receiver's Rx FIFO configuration options.
*
*****************************************************************************/

#include <windows.h>

#include "rx.h"
#include "align.h"
#include "arrow.h"
#include "fifo.h"
#include "resource.h"
#include "util_ctrl.h"



/* variables	*************************************************************/

static	fifo_data_t	_rx_data	=
{
	/* arrow_id				*/	IDC_RX_FIFO_ARROW,
	/* ctrl_id				*/	IDC_RX_FIFO,
	/* device				*/	NULL,
	/* overrun				*/
	{
	/* clear				*/	HPDI32_RX_OVERRUN_CLEAR,
	/* parm					*/	HPDI32_RX_OVERRUN,
	/* setting				*/	0,
	/* yes					*/	HPDI32_RX_OVERRUN_YES
	},
	/* overrun_color		*/	0,
	/* section_name			*/	"rx fifo",
	/* title				*/	"Receive",
	/* under_run			*/
	{
	/* clear				*/	HPDI32_RX_UNDER_RUN_CLEAR,
	/* parm					*/	HPDI32_RX_UNDER_RUN,
	/* setting				*/	0,
	/* yes					*/	HPDI32_RX_UNDER_RUN_YES
	},
	/* under_run_color		*/	0,
	/* which				*/	HPDI32_WHICH_RX,
	/* almost_empty			*/
	{
	/* defalt				*/	HPDI32_FIFO_ALMOST_EMPTY_DEFAULT,
	/* ignore				*/	0,
	/* setting				*/	0
	},
	/* almost_full	*/
	{
	/* defalt				*/	HPDI32_FIFO_ALMOST_FULL_DEFAULT,
	/* ignore				*/	0,
	/* setting				*/	0
	},
	/* changes				*/	0,
	/* dlg					*/	NULL,
	/* size					*/
	{
	/* defalt				*/	0,
	/* ignore				*/	0,
	/* setting				*/	0
	},
	/* size_known			*/	0,
	/* status				*/
	{
	/* defalt				*/	0,
	/* ignore				*/	0,
	/* setting				*/	0
	},
	/* was_open				*/	0
};



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Update the appearance of the Rx FIFO button.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, button_t* data)
{
	HWND	dlg		= GetParent(control);
	U32		overrun;
	U32		status;
	U32		underrun;

	if (data->device->handle)
	{
		status	= HPDI32_RX_OVERRUN__GET(data->device->handle, &overrun);
		error_status(dlg, IDC_INFO_TEXT, status);
		overrun	= (overrun == HPDI32_TX_OVERRUN_YES) ? TRUE : FALSE;
		status	= HPDI32_RX_UNDER_RUN__GET(data->device->handle, &underrun);
		error_status(dlg, IDC_INFO_TEXT, status);
		underrun	= (underrun == HPDI32_RX_UNDER_RUN_YES) ? TRUE : FALSE;
	}
	else
	{
		overrun		= FALSE;
		underrun	= FALSE;
	}

	fifo_gui_paint(control, data, &_rx_data, overrun, underrun);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_RX_FIFO,
		/* ref		*/	IDC_TX_FIFO,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_FIFO,
		/* ref		*/	IDC_RX_CLOCK,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_RX_FIFO_ARROW,
		/* ref		*/	IDC_TX_FIFO_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_FIFO_ARROW,
		/* ref		*/	IDC_RX_RECEIVE,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	MARGIN
		},
		{ 0, 0, 0, 0 }
	};

	ctrl_menu_set_text(dlg, IDC_RX_FIFO, "Rx FIFO");
	align_control_list(dlg, list);
	button_init(dlg, device, IDC_RX_FIFO);
	button_paint_function_set(dlg, IDC_RX_FIFO, _gui_paint);
	arrow_init(dlg, device, IDC_RX_FIFO_ARROW);
	arrow_type_set(dlg, IDC_RX_FIFO_ARROW, ARROW_LEFT_2);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	LRESULT	ret;

	ret	= fifo_service(dlg, device, SERVICE_REFRESH, (ULONG) &_rx_data, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	LRESULT	ret;

	ret	= fifo_service(dlg, device, SERVICE_WM_COMMAND, (ULONG) &_rx_data, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	rx_fifo_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx FIFO services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_fifo_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


