// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx_io.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Receiver I/O option.
*
*****************************************************************************/

#include <windows.h>

#include "rx.h"
#include "align.h"
#include "button.h"
#include "io.h"
#include "resource.h"
#include "util_ctrl.h"



/* variables	*************************************************************/

static	io_data_t	rx_data	=
{
	/* device			*/	NULL,
	/* section_name		*/	"rx io",
	/* title			*/	"Receive",
	/* which			*/	HPDI32_WHICH_RX,
	/* id_button		*/	IDC_RX_IO,
	/* api_buf_size		*/
	{
		/* control		*/	NULL,
		/* defalt		*/	0,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* app_buf			*/
	{
		/* control		*/	NULL,
		/* defalt		*/	0,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* sub_size			*/	0,
	/* changes			*/	0,
	/* data_size		*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_DATA_SIZE_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* dlg				*/	NULL,
	/* dma_ch_sel		*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* dma_ctrl_mode	*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_DMA_CONTROL_MODE_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* dma_pri			*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_DMA_PRIORITY_RX_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* mode				*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_MODE_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* pio_threshold	*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_PIO_THRESHOLD_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* single_cycle	*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_SINGLE_CYCLE_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* single_cycle_disable	*/	0,
	/* timeout			*/
	{
		/* control		*/	NULL,
		/* defalt		*/	HPDI32_IO_TIMEOUT_DEFAULT,
		/* ignore		*/	0,
		/* setting		*/	0
	},
	/* was_open			*/	FALSE
};



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_RX_IO,
		/* ref		*/	IDC_TX_IO,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_IO,
		/* ref		*/	IDC_RX_RECEIVE,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_RX_IO);
	return(0);
}



/*****************************************************************************
*
*	Function:	rx_io_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx I/O services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_io_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	LRESULT	ret;

	if (service == SERVICE_INIT)
		ret	= _service_init(dlg, device, wParam, lParam);
	else
		ret	= io_service(dlg, device, service, (ULONG) &rx_data, lParam);

	return(ret);
}


