// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_data.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Data configuration options.
*
*****************************************************************************/

#include <windows.h>

#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "cable.h"
#include "resource.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Tx Data control.
*
*	Arguments:
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, button_t* data)
{
	static const wave_t	list[]	=
	{
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	4
		},
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	0,
		/* count	*/	4,
		/* w1		*/	4,
		/* w2		*/	4
		},
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	-4,
		/* count	*/	1,
		/* w1		*/	4,
		/* w2		*/	4
		},
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	0
		},
		{ 0 }
	};

	button_arrow_t*	arrow;
	int				dx;
	int				dy;
	HWND			parent;
	HPEN			pen;
	U32				remote;
	U32				start;
	U32				stat;
	U32				status;
	U32				stop;
	int				x;
	int				y;

	// Row 1: left of arrow
	x	= data->inside2.rect.left
		+ data->arrow1.offset
		- MARGIN
		+ data->offset;
	y	= data->inside2.rect.top
		+ data->offset;

	x	-= button_text_paint(control, data, x, y, " ");

	if (data->arrow1.enable)
		arrow	= &data->arrow1;
	else
		arrow	= &data->arrow2;

	dx	= arrow->width / 2;
	dy	= (arrow->height / 2) * 2;
	y	= (y + (data->tmAscent / 2)) - (dy / 2);
	pen	= SelectObject(data->dc, arrow->pen);
	MoveToEx(data->dc, x, y, NULL);
	x	-= dx;
	LineTo(data->dc, x, y);
	y	+= dy;
	LineTo(data->dc, x, y );
	x	-= dx + 1;
	LineTo(data->dc, x, y);
	SelectObject(data->dc, pen);

	// Row 1: right of arrow
	x	= data->inside2.rect.left
		+ data->arrow1.offset
		+ data->arrow1.width
		+ MARGIN
		+ data->offset;
	y	= data->inside2.rect.top
		+ data->offset;

	x	+= button_text_paint(control, data, x, y, " ");
	button_wave_paint(data, x, y, list);

	// Row 2.
	x	= data->inside2.rect.left + data->offset;
	y	+= data->tmAscent;

	if (data->device->handle)
	{
		status	= GSC_SUCCESS;
		stat	= HPDI32_TX_AUTO_START__GET(data->device->handle, &start);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_TX_AUTO_STOP__GET(data->device->handle, &stop);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= HPDI32_TX_REMOTE_THROTTLE__GET(data->device->handle, &remote);
		status	= (status == GSC_SUCCESS) ? stat : status;
		parent	= GetParent(control);
		error_status(parent, IDC_INFO_TEXT, status);

		start	= (start == HPDI32_TX_AUTO_START_YES) ? FALSE : TRUE;
		stop	= (stop == HPDI32_TX_AUTO_STOP_YES) ? FALSE : TRUE;
		remote	= (remote == HPDI32_TX_REMOTE_THROTTLE_ENABLE) ? FALSE : TRUE;
	}
	else
	{
		start	= FALSE;
		stop	= FALSE;
		remote	= FALSE;
	}

	x	+= button_circle_paint(	control,
								data,
								x,
								y,
								data->device->paint.green.brush,
								(int) start);
	x	+= button_text_paint(control, data, x, y, " Auto Start   ");
	x	+= button_circle_paint(	control,
								data,
								x,
								y,
								data->device->paint.green.brush,
								(int) remote);
	x	+= button_text_paint(control, data, x, y, " Remote Throttle   ");
	x	+= button_circle_paint(	control,
								data,
								x,
								y,
								data->device->paint.red.brush,
								(int) stop);
	button_text_paint(control, data, x, y, " Auto Stop");
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_TX_DF,
		/* ref		*/	IDC_TX_CLOCK,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_DF,
		/* ref		*/	IDC_TX_FIFO_ARROW,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_DF_ARROW,
		/* ref		*/	IDC_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_DF_ARROW,
		/* ref		*/	IDC_TX_DF,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_TX_DF);
	button_arrow_direction_set(dlg, IDC_TX_DF, +1);
	button_paint_function_set(dlg, IDC_TX_DF, _gui_paint);
	arrow_init(dlg, device, IDC_TX_DF_ARROW);
	arrow_type_set(dlg, IDC_TX_DF_ARROW, ARROW_RIGHT_2);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[64];
	U32		cd;
	U32		clock;
	UINT	flags;
	size_t	len;
	char*	psz;
	U32		rate;
	U32		status;
	U32		tx;
	U32		width;

	if (device->handle)
	{
		status	= HPDI32_TX_CLOCK_DIVIDER__GET(device->handle, &cd);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_IO_DATA_SIZE__TX_GET(device->handle, &width);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_TX_ENABLE__GET(device->handle, &tx);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (cd < 1)
			clock	= device->oscillator;
		else
			clock	= device->oscillator / (cd * 2);

		switch (width)
		{
			default:
			case HPDI32_IO_DATA_SIZE_8_BITS:

				rate	= clock;
				strcpy(buf, "8-bits,");
				break;

			case HPDI32_IO_DATA_SIZE_16_BITS:

				rate	= clock * 2;
				strcpy(buf, "16-bits,");
				break;

			case HPDI32_IO_DATA_SIZE_32_BITS:

				rate	= clock * 4;
				strcpy(buf, "32-bits,");
				break;
		}

		psz	= buf + strlen(buf);
		format_u32(rate, 1000, psz);
		len	= strlen(psz);

		if (len > 4)
		{
			format_float((float) rate, psz);
			psz	= strchr(psz, ' ');

			if (psz)
			{
				psz[0]	= psz[1];
				psz[1]	= 0;
			}
		}

		strcat(buf, "B/S");
		ctrl_menu_set(dlg, IDC_TX_DF, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_DF_ARROW, CTRL_ENABLE);
		ctrl_menu_set_text(dlg, IDC_TX_DF_ARROW, buf);

		flags	= (tx == HPDI32_TX_ENABLE_NO) ? CTRL_DISABLE : CTRL_ENABLE;
		ctrl_menu_set(dlg, IDC_TX_DF_ARROW, flags);
	}
	else
	{
		ctrl_menu_set(dlg, IDC_TX_DF, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_TX_DF_ARROW, CTRL_DISABLE | CTRL_NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	LRESULT	ret;

	ret	= tx_data_flow_service(dlg, device, SERVICE_WM_COMMAND, wParam, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	tx_data_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Data services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_data_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


