// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_osc.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Oscillator configuration options.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "ini.h"
#include "preference.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	KEY_NAME			"oscillator"
#define	OSCILLATOR_K		1000
#define	SECTION_NAME		"gui"



/*****************************************************************************
*
*	Function:	_service_device_close
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_DEVICE_CLOSE service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_device_close(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	ini_u32_set(NULL, SECTION_NAME, KEY_NAME, device->oscillator);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_device_open
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_DEVICE_OPEN service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_device_open(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	U32	defalt;

	defalt	= PREF_OSCILLATOR_RATE__GET();
	ini_u32_get(NULL, SECTION_NAME, KEY_NAME, &device->oscillator, defalt);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_TX_OSCILLATOR,
		/* ref		*/	IDC_TX_ENABLE,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_OSCILLATOR,
		/* ref		*/	IDC_TX_ENABLE,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_TX_OSCILLATOR_ARROW,
		/* ref		*/	IDC_TX_OSCILLATOR,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B | ALIGN_L_R | ALIGN_W,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_TX_OSCILLATOR);
	arrow_init(dlg, device, IDC_TX_OSCILLATOR_ARROW);
	arrow_type_set(dlg, IDC_TX_OSCILLATOR_ARROW, ARROW_RIGHT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	size_t	len;
	char	osc[16];
	char*	psz;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_TX_OSCILLATOR, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_TX_OSCILLATOR_ARROW, CTRL_DISABLE | CTRL_NULL);
			break;
		}

		// The device is open.
		format_u32(device->oscillator, TX_OSCILLATOR_K, osc);
		len	= strlen(osc);

		if (len > 4)
		{
			format_float((float) device->oscillator, osc);
			psz	= strchr(osc, ' ');

			if (psz)
			{
				psz[0]	= psz[1];
				psz[1]	= 0;
			}
		}

		strcat(osc, "Hz");
		ctrl_menu_set(dlg, IDC_TX_OSCILLATOR, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_OSCILLATOR_ARROW, CTRL_ENABLE);
		ctrl_menu_set_text(dlg, IDC_TX_OSCILLATOR_ARROW, osc);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	LRESULT	ret;

	ret	= tx_clock_divider_service(dlg, device, SERVICE_WM_COMMAND, wParam, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	tx_oscillator_cd_idc_cd_oscillator
*
*	Purpose:
*
*		Respond to activity in the OSCILLATOR edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_oscillator_cd_idc_cd_oscillator(HWND dlg, WPARAM wParam, tx_cd_t* data)
{
	int	code	= HIWORD(wParam);
	U32	stat;
	U32	status;

	if ((code == EN_CHANGE) && (data->oscillator.ignore == FALSE))
	{
		status			= GSC_SUCCESS;
		data->changes	= TRUE;

		stat	= tx_oscillator_cd_dialog_read(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= tx_clock_divider_rate_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;

		error_status(dlg, 0, status);
	}
}



/*****************************************************************************
*
*	Function:	tx_oscillator_cd_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*		For this item, the value is retrieved from the device structure since
*		we can't query the device for the actual oscilator frequency.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

U32 tx_oscillator_cd_device_read(tx_cd_t* data)
{
	data->oscillator.setting	= data->device->oscillator;
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	tx_oscillator_cd_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*		For this item, the value is written to the device structure since we
*		can't control the device's oscilator frequency.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

U32 tx_oscillator_cd_device_write(tx_cd_t* data)
{
	data->device->oscillator	= data->oscillator.setting;
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	tx_oscillator_cd_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

U32 tx_oscillator_cd_dialog_read(HWND dlg, tx_cd_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_CD_OSCILLATOR_EDIT,
								OSCILLATOR_K,
								&data->oscillator.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	tx_oscillator_cd_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 tx_oscillator_cd_dialog_write(HWND dlg, tx_cd_t* data)
{
	char			buf[32];
	unsigned int	flags;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		format_u32(data->oscillator.setting, OSCILLATOR_K, buf);
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_CD_OSCILLATOR_TITLE, flags);
	ctrl_menu_set(dlg, IDC_CD_OSCILLATOR_EDIT, flags);
	ctrl_menu_set(dlg, IDC_CD_OSCILLATOR_INFO, flags);

	if (data->oscillator.ignore == FALSE)
	{
		data->oscillator.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_CD_OSCILLATOR_EDIT, buf);
		data->oscillator.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	tx_oscillator_cd_wm_initdialog
*
*	Purpose:
*
*		Process a the oscillator's portion of the Clock Divider dialog's
*		WM_INITDIALOG message.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_oscillator_cd_wm_initdialog(HWND dlg)
{
	char	buf[32];
	HWND	control;

	control	= GetDlgItem(dlg, IDC_CD_OSCILLATOR_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "Hz (1K = %d)", (int) OSCILLATOR_K);
	ctrl_menu_set_text(dlg, IDC_CD_OSCILLATOR_INFO, buf);
}



/*****************************************************************************
*
*	Function:	tx_oscillator_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Oscillator
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_oscillator_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function				*/
		{ SERVICE_DEVICE_CLOSE,	_service_device_close	},
		{ SERVICE_DEVICE_OPEN,	_service_device_open	},
		{ SERVICE_INIT,			_service_init			},
		{ SERVICE_REFRESH,		_service_refresh		},
		{ SERVICE_WM_COMMAND,	_service_wm_command		},
		{ SERVICE_END_LIST,		NULL					}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


