// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/version.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement the Version Data dialog.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "version.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"version data"



/* typedefs	*****************************************************************/

typedef struct
{
	device_t*	device;
	HWND		dlg;
} ver_t;



/* variables	*************************************************************/

static const struct
{
	U8			index;
	const char*	name;
	int			handle;
	int			ctrl_id;
} _ver_list[]	=
{
	{
		/* index	*/	GSC_VERSION_LIBRARY,
		/* name		*/	"API Library",
		/* hanndle	*/	0,
		/* ctrl_id	*/	IDC_VERSION_DATA_LABEL_1
	},
	{
		/* index	*/	GSC_VERSION_DRIVER,
		/* name		*/	"Device Driver",
		/* hanndle	*/	1,
		/* ctrl_id	*/	IDC_VERSION_DATA_LABEL_2
	}
};



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	char	buf[128];
	int		i;
	U32		status	= GSC_SUCCESS;
	char	ver[256];

	set_win_title(dlg, device, TRUE, TRUE, "Version Data");

	for (i = 0; i < (int) ARRAY_ELEMENTS(_ver_list); i++)
	{
		if ((_ver_list[i].handle) && (device->handle == 0))
		{
			status	= GSC_SUCCESS;
			sprintf(buf,
					"%s: Can't access without an open device.",
					_ver_list[i].name);
		}
		else
		{
			status	= hpdi32_version_get(	device->handle,
											_ver_list[i].index,
											ver,
											sizeof(ver));

			if (status != GSC_SUCCESS)
				sprintf(ver, "Access error: status 0x%lX", (long) status);

			sprintf(buf, "%s: %s", _ver_list[i].name, ver);
		}

		ctrl_menu_set_text(dlg, _ver_list[i].ctrl_id, buf);
	}

	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id						flags						*/
		{ -1,						COPY_EOL | COPY_UNDERLINE	},

		{ IDC_VERSION_DATA_LABEL_1,	COPY_EOL					},
		{ IDC_VERSION_DATA_LABEL_2,	COPY_EOL					},

		{ 0,						COPY_UNDERLINE				},
		{ 0,						COPY_EOL					},

		{ IDCANCEL,					COPY_EOL					}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	ver_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (ver_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	ver_t*	data;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (ver_t*) GET_WINDOW_USER_DATA(dlg);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	_refresh_window(dlg, data->device, NULL);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	ver_t	data;

	if (data.dlg)		/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.device	= device;
		data.dlg	= CreateDialogParam(device->instance,
										(LPCTSTR) IDD_VERSION_DATA,
										dlg,
										(DLGPROC) _dlg_proc,
										(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	version_data_service
*
*	Purpose:
*
*		Implement the common service entry point for the Version Data services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT version_data_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	LRESULT	ret;

	if (service == SERVICE_WM_COMMAND)
		ret	= _service_wm_command(dlg, device, wParam, lParam);
	else
		ret	= 0;

	return(ret);
}



