// HPDI32 - Application - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/xfer/rx.c $
// $Rev: 3630 $
// $Date: 2009-10-12 10:21:12 -0500 (Mon, 12 Oct 2009) $

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "hpdi32.h"
#include "main.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/*****************************************************************************
*
*	Function:	_show_rx_clock
*
*	Purpose:
*
*		Show the Rx clock rate.
*
*	Arguments:
*
*		clk			The clock rate is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _show_rx_clock(U32* clk)
{
	U32	status	= GSC_SUCCESS;

	printf("Rx Clock:       ");
	clk[0]	= io_clock;
	printf("%lu = ", (long) io_clock);
	format_value(clk[0], "Hz");

	if (clk[0] < 1)
	{
		printf("\nERROR: The clock rate is too low.\n");
		status	= GSC_FAILED;
	}

	printf("\n");
	return(status);
}



/*****************************************************************************
*
*	Function:	_show_flow_control
*
*	Purpose:
*
*		Show the cable's Rx Flow Control confg\iguration.
*
*	Arguments:
*
*		device		The handle used to access the device under test.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _show_flow_control(void* device)
{
	unsigned long	fv;
	unsigned long	lv;
	U32				status;
	unsigned long	sv;

	for (;;)	// A convenience loop.
	{
		printf("Flow Control:   Rx Clock");

		status	= HPDI32_CABLE_COMMAND_MODE__FV_GET(device, &fv);
		if (test_status(status, "HPDI32_CABLE_COMMAND_MODE__FV_GET"))	break;

		status	= HPDI32_CABLE_COMMAND_MODE__LV_GET(device, &lv);
		if (test_status(status, "HPDI32_CABLE_COMMAND_MODE__LV_GET"))	break;

		status	= HPDI32_CABLE_COMMAND_MODE__SV_GET(device, &sv);
		if (test_status(status, "HPDI32_CABLE_COMMAND_MODE__SV_GET"))	break;

		if (fv == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
			printf(", Frame Valid");

		if (lv == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
			printf(", Line Valid");

		if (sv == HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
			printf(", Status Valid");

		printf("\n");
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_rx_data
*
*	Purpose:
*
*		Receive data per the existing parameters.
*
*	Arguments:
*
*		device		The handle used to access the device under test.
*
*		src			The source buffer to transmit.
*
*		bytes		The size of the buffer in bytes.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _rx_data(void* device, void* src, U32 bytes)
{
	U32	block;
	U32	status;
	U32	xfer;

	for (;;)	// A convenience loop.
	{
		printf("Receiving:      ");
		status	= HPDI32_RX_ENABLE__YES(device);
		if (test_status(status, "HPDI32_RX_ENABLE__YES"))	break;

		for (block = 0; block < block_count; block++)
		{
			status	= hpdi32_read(device, src, bytes, &xfer);

			if (status == GSC_WAIT_TIMEOUT)
				;
			else if (test_status(status, "hpdi32_read"))	break;

			if (status == GSC_WAIT_TIMEOUT)
				printf("T");
			else
				printf(".");

			if (((block % 50) == 49) || ((block + 1) == block_count))
				printf(" %ld", (long) block + 1);

			if ((block % 50) == 49)
				printf("\n                ");
		}

		if ((status == GSC_SUCCESS) || (status == GSC_WAIT_TIMEOUT))
			printf(" Done\n");

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	rx_process
*
*	Purpose:
*
*		Implement the data reception process.
*
*	Arguments:
*
*		device		The handle used to access the device under test.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 rx_process(void* device)
{
	unsigned long	begin;
	double			bytes;
	U32				clk;
	void*			ptr;
	const char*		psz;
	unsigned long	size;
	struct tm*		stm;
	U32				status;
	time_t			tt;
	U32				u32;

	for (;;)	// A convenience loop.
	{
		printf("DATA RECEPTION:\n");

		status	= tx_rx_tristate(device);
		if (status != GSC_SUCCESS)	break;

		status	= _show_rx_clock(&clk);
		if (status != GSC_SUCCESS)	break;

		status	= show_data_set(device, HPDI32_WHICH_RX, &size, &bytes);
		if (status != GSC_SUCCESS)	break;

		status	= show_data_rate(HPDI32_WHICH_RX, clk, size);
		if (status != GSC_SUCCESS)	break;

		status	= estimate_duration(clk, size, bytes);
		if (status != GSC_SUCCESS)	break;

		status	= _show_flow_control(device);
		if (status != GSC_SUCCESS)	break;

		printf("Begin Time:     ");
		time(&tt);
		stm	= localtime(&tt);
		psz	= asctime(stm);
		printf("%s", psz);
		begin	= gsc_os_time_delta_ms();

		u32		= size * block_samples;
		ptr		= malloc(u32);

		if (ptr)
		{

			status	= _rx_data(device, ptr, u32);
			free(ptr);
		}
		else
		{
			printf("ERROR: malloc(%ld) failure.\n", (long) u32);
			status	= GSC_FAILED;
		}

		actual_duration((long) begin);
		break;
	}

	return(status);
}


